package it.cnr.isti.epasmed.config;

import java.sql.SQLException;
import java.util.HashMap;

import javax.sql.DataSource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.orm.jpa.EntityManagerFactoryBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.context.annotation.Profile;
import org.springframework.core.env.Environment;
import org.springframework.core.env.Profiles;
import org.springframework.data.jpa.repository.config.EnableJpaAuditing;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;
import org.springframework.orm.jpa.JpaTransactionManager;
import org.springframework.orm.jpa.LocalContainerEntityManagerFactoryBean;
import org.springframework.orm.jpa.vendor.HibernateJpaVendorAdapter;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.annotation.EnableTransactionManagement;

import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;

import io.github.jhipster.config.JHipsterConstants;
import io.github.jhipster.config.h2.H2ConfigurationHelper;
import it.cnr.isti.epasmed.config.ApplicationProperties.DataSourceProperties;
import it.cnr.isti.epasmed.config.ApplicationProperties.DatasourceEpasmed;
import it.cnr.isti.epasmed.config.ApplicationProperties.HikariProperties;
import it.cnr.isti.epasmed.config.ApplicationProperties.LiquibaseProperties;
import liquibase.integration.spring.SpringLiquibase;

@Configuration
@EnableJpaRepositories(basePackages = "it.cnr.isti.epasmed.repository", entityManagerFactoryRef = "epasMedEntityManagerFactory", transactionManagerRef = "epasMedTransactionManager")
@EnableJpaAuditing(auditorAwareRef = "springSecurityAuditorAware")
@EnableTransactionManagement
public class EpasMedDataSourceConfiguration {

	private final Logger log = LoggerFactory.getLogger(EpasMedDataSourceConfiguration.class);

	private final Environment env;
	private final ApplicationProperties applicationProperties;

	public EpasMedDataSourceConfiguration(Environment env, ApplicationProperties applicationProperties) {
		this.env = env;
		this.applicationProperties = applicationProperties;
	}

	/**
	 * Open the TCP port for the H2 database, so it is available remotely.
	 *
	 * @return the H2 database TCP server.
	 * @throws SQLException if the server failed to start.
	 */
	@Bean(initMethod = "start", destroyMethod = "stop")
	@Profile(JHipsterConstants.SPRING_PROFILE_DEVELOPMENT)
	public Object h2TCPServer() throws SQLException {
		String port = getValidPortForH2();
		log.debug("H2 database is available on port {}", port);
		return H2ConfigurationHelper.createServer(port);
	}

	private String getValidPortForH2() {
		int port = Integer.parseInt(env.getProperty("server.port"));
		if (port < 10000) {
			port = 10000 + port;
		} else {
			if (port < 63536) {
				port = port + 2000;
			} else {
				port = port - 2000;
			}
		}
		return String.valueOf(port);
	}

	@Bean
	@Primary
	public DatasourceEpasmed epasMedDataSourceProperties() {
		return applicationProperties.getDatasourceEpasmed();
	}

	@Bean
	@Primary
	public DataSource epasMedDataSource() {
		log.info("Setup ePASMedDataSource");
		DataSourceProperties dsp = epasMedDataSourceProperties().getDatasource();
		log.info("Properties: {}",dsp);
		HikariProperties hikariProperties  = epasMedDataSourceProperties().getHikari();
		log.info("Hikari: {}",hikariProperties);
		
		HikariConfig config = new HikariConfig();
		config.setJdbcUrl(dsp.getUrl());
		config.setUsername(dsp.getUsername());
		config.setPassword(dsp.getPassword());
		config.addDataSourceProperty("cachePrepStmts", hikariProperties.isCachePrepStmts());
		config.addDataSourceProperty("prepStmtCacheSize", hikariProperties.getPrepStmtCacheSize());
		config.addDataSourceProperty("prepStmtCacheSqlLimit", hikariProperties.getPrepStmtCacheSqlLimit());
		config.setAutoCommit(hikariProperties.isAutoCommit());
		config.setPoolName(hikariProperties.getPoolName());
		config.setMaximumPoolSize(hikariProperties.getMaximumPoolSize());
		log.info("HikariConfig epasmed: {}", dsp.getUrl());
		HikariDataSource ds = new HikariDataSource(config);
		return ds;
	}

	@Bean(name = "epasMedEntityManagerFactory")
	@Primary
	public LocalContainerEntityManagerFactoryBean epasMedEntityManagerFactory(EntityManagerFactoryBuilder builder) {
		LocalContainerEntityManagerFactoryBean em = builder.dataSource(epasMedDataSource()).persistenceUnit("epasMedPU")
				.packages("it.cnr.isti.epasmed.domain").build();

		HibernateJpaVendorAdapter vendorAdapter = new HibernateJpaVendorAdapter();
		// vendorAdapter.setDatabase(Database.DB2);
		vendorAdapter.setShowSql(true);
		vendorAdapter.setDatabasePlatform(epasMedDataSourceProperties().getDatasource().getPlatform());
		

		em.setJpaVendorAdapter(vendorAdapter);
		HashMap<String, Object> properties = new HashMap<>();
		properties.put("hibernate.jdbc.time_zone", "UTC");
		properties.put("hibernate.id.new_generator_mappings", true);
		properties.put("hibernate.connection.provider_disables_autocommit", true);
		properties.put("hibernate.cache.use_second_level_cache", false);
		properties.put("hibernate.cache.use_query_cache", false);
		properties.put("hibernate.generate_statistics", false);
		// modify batch size as necessary
		properties.put("hibernate.jdbc.batch_size", 25);
		properties.put("hibernate.order_inserts", true);
		properties.put("hibernate.order_updates", true);
		properties.put("hibernate.jdbc.lob.non_contextual_creation", true);
		properties.put("hibernate.query.fail_on_pagination_over_collection_fetch", true);
		properties.put("hibernate.query.in_clause_parameter_padding", true);

		// Naming props
		properties.put("hibernate.ddl-auto", "none");
		properties.put("hibernate.physical_naming_strategy",
				"org.springframework.boot.orm.jpa.hibernate.SpringPhysicalNamingStrategy");
		properties.put("hibernate.implicit_naming_strategy",
				"org.springframework.boot.orm.jpa.hibernate.SpringImplicitNamingStrategy");
		//org.hibernate.boot.model.naming.ImplicitNamingStrategyLegacyJpaImpl

		
		properties.put("hibernate.hbm2ddl.auto","none");
		properties.put("hibernate.dialect", env.getProperty("hibernate.dialect"));
		em.setJpaPropertyMap(properties);
		return em;
	}

	@Bean
	@Primary
	public PlatformTransactionManager epasMedTransactionManager(
			final @Qualifier("epasMedEntityManagerFactory") LocalContainerEntityManagerFactoryBean epasMedEntityManagerFactory) {
		return new JpaTransactionManager(epasMedEntityManagerFactory.getObject());
	}

	@Bean
	public LiquibaseProperties epasMedLiquibaseProperties() {
		return applicationProperties.getDatasourceEpasmed().getLiquibase();
	}

	@Bean
	public SpringLiquibase epasMedLiquibase() {
		return liquibase(epasMedLiquibaseProperties(), epasMedDataSource());
	}

	private SpringLiquibase liquibase(LiquibaseProperties epasMedLiquibaseProperties, DataSource epasMedDataSource) {
		log.debug("Create EpasMed Liquibase DataSource");
		SpringLiquibase liquibase = new SpringLiquibase();
		liquibase.setDataSource(epasMedDataSource);
		// SpringLiquibaseUtil.createAsyncSpringLiquibase(env,
		// asyncConfiguration.getAsyncExecutor(), liquibaseDataSource,
		// epasMedLiquibaseProperties,
		// epasMedDataSource, epasMedDataSourceProperties);
		log.debug("Set params");
		liquibase.setContexts(epasMedLiquibaseProperties.getContexts());
		liquibase.setChangeLog(epasMedLiquibaseProperties.getChangeLog());
		liquibase.setDefaultSchema(epasMedLiquibaseProperties.getDefaultSchema());
		liquibase.setLiquibaseSchema(epasMedLiquibaseProperties.getLiquibaseSchema());
		liquibase.setLiquibaseTablespace(epasMedLiquibaseProperties.getLiquibaseTablespace());
		liquibase.setDatabaseChangeLogLockTable(epasMedLiquibaseProperties.getDatabaseChangeLogLockTable());
		liquibase.setDatabaseChangeLogTable(epasMedLiquibaseProperties.getDatabaseChangeLogTable());
		liquibase.setDropFirst(epasMedLiquibaseProperties.isDropFirst());
		liquibase.setLabels(epasMedLiquibaseProperties.getLabels());
		liquibase.setChangeLogParameters(epasMedLiquibaseProperties.getParameters());
		liquibase.setRollbackFile(epasMedLiquibaseProperties.getRollbackFile());
		liquibase.setTestRollbackOnUpdate(epasMedLiquibaseProperties.isTestRollbackOnUpdate());
		if (env.acceptsProfiles(Profiles.of(JHipsterConstants.SPRING_PROFILE_NO_LIQUIBASE))) {
			liquibase.setShouldRun(false);
		} else {
			liquibase.setShouldRun(epasMedLiquibaseProperties.isEnabled());
			log.debug("Configuring epasMed Liquibase");
		}
		log.debug("Created epasMed Libquibase");
		return liquibase;
	}

}
