package it.cnr.isti.epasmed.epas.service;

import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import it.cnr.isti.epasmed.epas.dto.EPASPersonWorkingTimeDTO;
import it.cnr.isti.epasmed.epas.model.EPASContracts;
import it.cnr.isti.epasmed.epas.model.EPASPersons;
import it.cnr.isti.epasmed.epas.model.EPASWorkingTimeForPerson;
import it.cnr.isti.epasmed.epas.model.EPASWorkingTimeTypeDays;
import it.cnr.isti.epasmed.epas.model.EPASWorkingTimeTypes;

@Service
public class EPASPersonWorkingTimeService {

	private static final Logger logger = LoggerFactory.getLogger(EPASPersonWorkingTimeService.class);
	private final DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");

	@Autowired
	EPASWorkingTimeTypesService epasWorkingTimeTypesService;

	@Autowired
	EPASPersonsService epasPersonService;

	@Autowired
	EPASContractsService epasContranctsService;

	public EPASPersonWorkingTimeDTO getByPersonId(String personId) {

		List<EPASContracts> listContracts = null;
		try {
			listContracts = epasContranctsService.getByPersonId(personId);
		} catch (Exception e) {
			logger.error("Error retrieving contracts for person id: {}", personId);
			logger.error(e.getLocalizedMessage(), e);
			return null;
		}

		if (listContracts == null || listContracts.isEmpty()) {
			logger.error("There is no contract for this person Id: {}", personId);
			return null;
		}

		EPASPersonWorkingTimeDTO pwtDTO = getPersonWorkingTimeDTO(personId, listContracts);
		return pwtDTO;
	}

	public EPASPersonWorkingTimeDTO getByPersonFiscalCode(String fc) {
		EPASPersons epasPerson = epasPersonService.getByFiscalCode(fc);
		if (epasPerson == null || epasPerson.getId() == null || epasPerson.getId().isEmpty()) {
			logger.error("There is no Person for fiscal code: {}", fc);
			return null;
		}
		logger.debug("EPASPerson: {}", epasPerson);

		List<EPASContracts> listContracts = null;
		try {
			listContracts = epasContranctsService.getByPersonFiscalcode(fc);
		} catch (Exception e) {
			logger.error("Error retrieving contracts for fiscal code: {}", fc);
			logger.error(e.getLocalizedMessage(), e);
			return null;
		}

		if (listContracts == null || listContracts.isEmpty()) {
			logger.error("There is no contract for this person Id: {}", epasPerson.getId());
			return null;
		}
		EPASPersonWorkingTimeDTO pwtDTO = getPersonWorkingTimeDTO(epasPerson.getId(), listContracts);
		return pwtDTO;
	}

	private EPASPersonWorkingTimeDTO getPersonWorkingTimeDTO(String personId, List<EPASContracts> listContracts) {
		String lastContractId = null;
		LocalDate lastContractDate = null;
		LocalDate currentContractDate = null;
		for (EPASContracts contract : listContracts) {
			currentContractDate = LocalDate.parse(contract.getBeginDate(), formatter);
			if (lastContractDate == null || currentContractDate.isAfter(lastContractDate)) {
				lastContractDate = currentContractDate;
				lastContractId = contract.getId();
			}
		}

		if (lastContractId == null || lastContractId.isEmpty()) {
			logger.error("There is no last contract for this person id: {}", personId);
			return null;
		}
		logger.debug("Found Last Contract Id: {}", lastContractId);

		EPASContracts lastContract = epasContranctsService.getById(lastContractId);

		if (lastContract == null) {
			logger.error("There is no last contract by Id: {}", lastContractId);
			return null;
		}
		if (lastContract.getWorkingTimeTypes() == null || lastContract.getWorkingTimeTypes().length < 0) {
			logger.error("There is no last Working Time Type for this Contract Id: {}", lastContractId);
			return null;
		}

		String lastWTPId = null;
		LocalDate lastWTPBeginDate = null;
		String lastWTPBeginDateS = null;
		String lastWTPEndDate = null;
		LocalDate currentWTPBeginDate = null;
		for (EPASWorkingTimeForPerson wtp : lastContract.getWorkingTimeTypes()) {
			currentWTPBeginDate = LocalDate.parse(wtp.getBeginDate(), formatter);
			if (lastWTPBeginDate == null || currentWTPBeginDate.isAfter(lastWTPBeginDate)) {
				lastWTPBeginDate = currentWTPBeginDate;
				lastWTPBeginDateS = wtp.getBeginDate();
				lastWTPEndDate = wtp.getEndDate();
				lastWTPId = wtp.getWorkingTimeType().getId();
			}
		}

		if (lastWTPId == null) {
			logger.error("There is no last Working Time Type by Id: {}", lastWTPId);
			return null;
		}
		logger.debug("Retrieving Working Time Type by Id: {}", lastWTPId);

		EPASWorkingTimeTypes wtt = epasWorkingTimeTypesService.getById(lastWTPId);
		if (wtt == null) {
			logger.error("There is no Working Time Type by Id: {}", lastWTPId);
			return null;
		}

		if (wtt.getWorkingTimeTypeDays().length < 1) {
			logger.error("There is no last Working Time Type Days by Id: {}", lastWTPId);
			return null;
		}

		LinkedHashMap<Integer, Integer> workingTimeByDays = new LinkedHashMap<Integer, Integer>();
		int percentuale = 0;
		for (EPASWorkingTimeTypeDays wtd : wtt.getWorkingTimeTypeDays()) {
			boolean isHoliday = Boolean.parseBoolean(wtd.getHoliday());
			try {
				int dayOfWeek = Integer.parseInt(wtd.getDayOfWeek());

				if (isHoliday) {
					workingTimeByDays.put(dayOfWeek, 0);
				} else {
					int workingTime = Integer.parseInt(wtd.getWorkingTime());
					workingTimeByDays.put(dayOfWeek, workingTime);
					percentuale += workingTime;
				}
			} catch (NumberFormatException e) {
				logger.error("Invalid working time by days: " + e.getLocalizedMessage(), e);
			}
		}

		percentuale = (100 * percentuale) / 2160;
		// TODO
		// turno e le ore turno.
		EPASPersonWorkingTimeDTO pwtDTO = new EPASPersonWorkingTimeDTO(lastContractId, personId,
				lastContract.getPerson().getFiscalCode(), lastWTPBeginDateS, lastWTPEndDate, wtt.getDescription(),
				workingTimeByDays.get(DayOfWeek.MONDAY.getValue()), workingTimeByDays.get(DayOfWeek.TUESDAY.getValue()),
				workingTimeByDays.get(DayOfWeek.WEDNESDAY.getValue()),
				workingTimeByDays.get(DayOfWeek.THURSDAY.getValue()),
				workingTimeByDays.get(DayOfWeek.FRIDAY.getValue()),
				workingTimeByDays.get(DayOfWeek.SATURDAY.getValue()), percentuale, "NO", 0, "SI", "NO");
		return pwtDTO;
	}

	
	public List<EPASPersonWorkingTimeDTO> getList(String officeId) {
		List<EPASPersons> epasPersons=epasPersonService.getList(officeId);
		if(epasPersons==null||epasPersons.isEmpty()) {
			logger.error("No persons found for office id: {}",officeId);
			return null;
		}
		
		List<EPASPersonWorkingTimeDTO> pwtDTOList=new LinkedList<>();
		for(EPASPersons epasPerson:epasPersons) {
			EPASPersonWorkingTimeDTO pwtDTO=getByPersonId(epasPerson.getId());
			if(pwtDTO!=null) {
				pwtDTOList.add(pwtDTO);
			} else {
				logger.error("Not exists Working Time for Person id: {}", epasPerson.getId());
			}
		}
		return pwtDTOList;
	}
	
}
