package it.cnr.isti.epasmed.service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.convert.ConversionService;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import it.cnr.isti.epasmed.domain.TimeCardsReporting;
import it.cnr.isti.epasmed.repository.TimeCardsReportingRepository;

/**
 * Service class for managing users.
 */
@Service
@Transactional("epasMedTransactionManager")
public class TimeCardsReportingService {

	private final Logger log = LoggerFactory.getLogger(TimeCardsReportingService.class);

	private final TimeCardsReportingRepository timeCardsReportingRepository;

	@Autowired
	ConversionService conversionService;

	// @Autowired
	// FormattingConversionService conversionDateTimeService;

	public TimeCardsReportingService(TimeCardsReportingRepository timeCardsReportingRepository) {
		this.timeCardsReportingRepository = timeCardsReportingRepository;
	}

	@Transactional(value = "epasMedTransactionManager", readOnly = true)
	public Page<TimeCardsReporting> getAllTimeCardsReporting(Pageable pageable) {
		log.debug("TimeCardsReportingService  getAllTimeCardsReporting(): {}", pageable);
		timeCardsReportingRepository.findAll();
		return timeCardsReportingRepository.findAll(pageable);
	}

	@Transactional(value = "epasMedTransactionManager", readOnly = true)
	public List<TimeCardsReporting> getAllTimeCardsReporting() {
		log.debug("TimeCardsReportingService  getAllTimeCardsReporting()");
		return timeCardsReportingRepository.findAll();
	}

	@Transactional(value = "epasMedTransactionManager", readOnly = true)
	public Optional<TimeCardsReporting> getTimeCardsReportingById(Long id) {
		return timeCardsReportingRepository.findById(id);
	}

	@Transactional(value = "epasMedTransactionManager", readOnly = true)
	public Optional<TimeCardsReporting> getLastTimeCardsReporting() {
		return timeCardsReportingRepository.findFirstByOrderByIdDesc();
	}
	
	/**
	 * Create a new TimeCards Reporting 
	 * 
	 * @param timeCardsReportingsDTO
	 * @return TimeCardsReporting
	 */
	public TimeCardsReporting createTimeCardsReporting(TimeCardsReporting timeCardsReportingsDTO) {
		log.debug("Creating new TimeCardsReporting: {}", timeCardsReportingsDTO);
		TimeCardsReporting timeCardsReporting = new TimeCardsReporting();
		timeCardsReporting.setYear(timeCardsReportingsDTO.getYear());
		timeCardsReporting.setMonth(timeCardsReportingsDTO.getMonth());
		timeCardsReporting.setIdFlusso(timeCardsReportingsDTO.getIdFlusso());
		timeCardsReporting.setLastUpdate(timeCardsReportingsDTO.getLastUpdate());
		timeCardsReportingRepository.save(timeCardsReporting);

		log.debug("Created TimeCardsReporting: {}", timeCardsReporting);
		return timeCardsReporting;
	}

	/**
	 * Update all information for a specific timecards reporting.
	 * 
	 * @param timeCardsReporting
	 * @return
	 */
	public Optional<TimeCardsReporting> updateTimeCardsReporting(TimeCardsReporting timeCardsReporting) {
		Optional<TimeCardsReporting> tcR = timeCardsReportingRepository.findById(timeCardsReporting.getId());
		if (tcR.isPresent()) {
			TimeCardsReporting timeCardsR = tcR.get();
			timeCardsR.setYear(timeCardsReporting.getYear());
			timeCardsR.setMonth(timeCardsReporting.getMonth());
			timeCardsR.setIdFlusso(timeCardsReporting.getIdFlusso());
			timeCardsR.setLastUpdate(timeCardsReporting.getLastUpdate());
			timeCardsR = timeCardsReportingRepository.save(timeCardsR);
			log.debug("Changed Information for TimeCardsReporting: {}", timeCardsR);
			return Optional.of(timeCardsR);
		} else {
			return tcR;
		}
	}

	public void deleteTimeCardsReporting(Long id) {
		timeCardsReportingRepository.findById(id).ifPresent(tcR -> {
			timeCardsReportingRepository.delete(tcR);
			log.debug("Deleted TimeCardsReporting: {}", tcR);
		});
	}

	/**
	 * Update TimeCardsReporting information (id, year, month, id flusso, last update).
	 * 
	 * @param id         Identify
	 * @param year       Year
	 * @param month      Month
	 * @param idFlusso   Id Flusso
	 * @param lastUpdate Last Update
	 */
	public void updateTimeCardsReporting(String id, Integer year, Integer month, String idFlusso, String lastUpdate) {
		LocalDateTime lastU = conversionService.convert(lastUpdate, LocalDateTime.class);
		// LocalDateTime lastU=conversionDateTimeService.convert(lastUpdate,
		// LocalDateTime.class);

		Optional.of(timeCardsReportingRepository.findById(Long.valueOf(id))).filter(Optional::isPresent).map(Optional::get)
				.map(tcR -> {
					tcR.setYear(year);
					tcR.setMonth(month);
					tcR.setIdFlusso(Long.valueOf(idFlusso));
					tcR.setLastUpdate(lastU);
					log.debug("Changed Information for TimeCardsReporting: {}", tcR);
					return tcR;
				});
	}

}
