package it.cnr.isti.epasmed.web.rest;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;

import io.github.jhipster.web.util.HeaderUtil;
import io.github.jhipster.web.util.PaginationUtil;
import io.github.jhipster.web.util.ResponseUtil;
import it.cnr.isti.epasmed.config.Constants;
import it.cnr.isti.epasmed.domain.Leaves;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;
import it.cnr.isti.epasmed.service.LeavesService;
import it.cnr.isti.epasmed.web.rest.errors.BadRequestAlertException;

/**
 * REST controller for managing Leaves.
 * <p>
 * This class accesses the {@link Leaves} entity.
 * 
 * 
 */
@RestController
@RequestMapping("/api")
public class LeavesResource {
	private static final List<String> ALLOWED_ORDERED_PROPERTIES = Collections
			.unmodifiableList(Arrays.asList("id", "idPersona", "cf", "codiceAssenzaDescrizione", "dataInizio",
					"dataFine","dataMod"));
	

	private final Logger log = LoggerFactory.getLogger(LeavesResource.class);

	@Value("${jhipster.clientApp.name}")
	private String applicationName;

	private final LeavesService leavesService;

	public LeavesResource(LeavesService leavesService) {
		this.leavesService = leavesService;

	}

	/**
	 * {@code GET /leaves} : get all leaves.
	 *
	 * @param pageable the pagination information.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         all leaves.
	 */
	@GetMapping("/leaves")
	public ResponseEntity<List<Leaves>> getAllLeaves(Pageable pageable) {
		log.info("REST request to read Leaves : {}", pageable);

		if (!onlyContainsAllowedProperties(pageable)) {
			return ResponseEntity.badRequest().build();
		}

		final Page<Leaves> page = leavesService.getAllLeaves(pageable);
		HttpHeaders headers = PaginationUtil
				.generatePaginationHttpHeaders(ServletUriComponentsBuilder.fromCurrentRequest(), page);
		return new ResponseEntity<>(page.getContent(), headers, HttpStatus.OK);
	}

	private boolean onlyContainsAllowedProperties(Pageable pageable) {
		return pageable.getSort().stream().map(Sort.Order::getProperty).allMatch(ALLOWED_ORDERED_PROPERTIES::contains);
	}

	/**
	 * {@code POST  /leaves} : Creates a new Leaves.
	 * <p>
	 *
	 * @param leavesDTO Leave to create.
	 * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with
	 *         body the new Leave, or with status {@code 400 (Bad Request)} if
	 *         the id is already in use.
	 * @throws URISyntaxException if the Location URI syntax is incorrect.
	 */
	@PostMapping("/leaves")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<Leaves> createLeaves(@Valid @RequestBody Leaves leavesDTO) throws URISyntaxException {
		log.info("REST request to save Leaves : {}", leavesDTO);

		if (leavesDTO.getId() != null) {
			throw new BadRequestAlertException("A new leave cannot already have an ID", "Leaves", "idexists");
		} else {
			Leaves leave = leavesService.createLeaves(leavesDTO);
			return ResponseEntity.created(new URI("/api/leaves/" + leave.getId())).headers(HeaderUtil
					.createAlert(applicationName, "A leave is created with identifier " + leave.getId(), leave.getCf()))
					.body(leave);
		}
	}

	/**
	 * {@code PUT /leaves} : Updates leaves.
	 *
	 * @param leavesDTO the leave to update.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the updated leave.
	 * 
	 */
	@PutMapping("/leaves")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<Leaves> updateLeaves(@Valid @RequestBody Leaves leavesDTO) {
		log.debug("REST request to update Leaves : {}", leavesDTO);
		Optional<Leaves> updatedLeaves = leavesService.updateLeaves(leavesDTO);

		return ResponseUtil.wrapOrNotFound(updatedLeaves, HeaderUtil.createAlert(applicationName,
				"A leave is updated with identifier " + leavesDTO.getId(), leavesDTO.getCf()));
	}

	/**
	 * {@code GET /leaves/:id} : get the "id" of leave.
	 *
	 * @param id the id of the leave to find.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the leave, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/leaves/{id:" + Constants.VALID_NUMBER_REGEX + "}")
	public ResponseEntity<Leaves> getLeaves(@PathVariable String id) {
		log.debug("REST request to get Leave : {}", id);
		return ResponseUtil.wrapOrNotFound(leavesService.getLeavesById(Integer.valueOf(id)));

	}

	/**
	 * {@code DELETE /leaves/:id} : delete the "id" Leave.
	 *
	 * @param id the id of the leave to delete.
	 * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
	 */
	@DeleteMapping("/leaves/{id:" + Constants.VALID_NUMBER_REGEX + "}")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<Void> deleteLeaves(@PathVariable String id) {
		log.debug("REST request to delete Leaves id: {}", id);
		leavesService.deleteLeaves(Integer.valueOf(id));
		return ResponseEntity.noContent()
				.headers(HeaderUtil.createAlert(applicationName, "A Leave is deleted with identifier " + id, id))
				.build();
	}

}
