package it.cnr.isti.epasmed.web.rest;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;

import io.github.jhipster.web.util.HeaderUtil;
import io.github.jhipster.web.util.PaginationUtil;
import io.github.jhipster.web.util.ResponseUtil;
import it.cnr.isti.epasmed.config.Constants;
import it.cnr.isti.epasmed.domain.Leaves;
import it.cnr.isti.epasmed.domain.PersOrario;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;
import it.cnr.isti.epasmed.service.PersOrarioService;
import it.cnr.isti.epasmed.web.rest.errors.BadRequestAlertException;

/**
 * REST controller for managing PersOrario.
 * <p>
 * This class accesses the {@link Leaves} entity.
 * 
 * 
 */
@RestController
@RequestMapping("/api")
public class PersOrarioResource {
	private static final List<String> ALLOWED_ORDERED_PROPERTIES = Collections
			.unmodifiableList(Arrays.asList("id", "idPersona", "cf", "dal",
					"al","descrizione","dataMod"));
	

	private final Logger logger = LoggerFactory.getLogger(PersOrarioResource.class);

	@Value("${jhipster.clientApp.name}")
	private String applicationName;

	private final PersOrarioService persOrarioService;

	public PersOrarioResource(PersOrarioService persOrarioService) {
		this.persOrarioService = persOrarioService;

	}

	/**
	 * {@code GET /persorario} : get all pers orario.
	 *
	 * @param pageable the pagination information.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         all pers orario.
	 */
	@GetMapping("/persorario")
	public ResponseEntity<List<PersOrario>> getAllPersOrario(Pageable pageable) {
		logger.info("REST request to read PersOrario : {}", pageable);

		if (!onlyContainsAllowedProperties(pageable)) {
			return ResponseEntity.badRequest().build();
		}

		final Page<PersOrario> page = persOrarioService.getAllPersOrario(pageable);
		HttpHeaders headers = PaginationUtil
				.generatePaginationHttpHeaders(ServletUriComponentsBuilder.fromCurrentRequest(), page);
		return new ResponseEntity<>(page.getContent(), headers, HttpStatus.OK);
	}

	private boolean onlyContainsAllowedProperties(Pageable pageable) {
		return pageable.getSort().stream().map(Sort.Order::getProperty).allMatch(ALLOWED_ORDERED_PROPERTIES::contains);
	}

	/**
	 * {@code POST  /persorario} : Creates a new pers orario.
	 * <p>
	 *
	 * @param persOrarioDTO PersOrario to create.
	 * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with
	 *         body the new PersOrario, or with status {@code 400 (Bad Request)} if
	 *         the id is already in use.
	 * @throws URISyntaxException if the Location URI syntax is incorrect.
	 */
	@PostMapping("/persorario")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<PersOrario> createPersOrario(@Valid @RequestBody PersOrario persOrarioDTO) throws URISyntaxException {
		logger.info("REST request to save PersOrario : {}", persOrarioDTO);

		if (persOrarioDTO.getId() != null) {
			throw new BadRequestAlertException("A new pers orario cannot already have an ID", "PersOrario", "idexists");
		} else {
			PersOrario persOrario = persOrarioService.createPersOrario(persOrarioDTO);
			return ResponseEntity.created(new URI("/api/persorario/" + persOrario.getId())).headers(HeaderUtil
					.createAlert(applicationName, "A pers orario is created with identifier " + persOrario.getId(), persOrario.getCf()))
					.body(persOrario);
		}
	}

	/**
	 * {@code PUT /persorario} : Updates pers orario.
	 *
	 * @param persOrarioDTO the pers orario to update.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the updated pers orario.
	 * 
	 */
	@PutMapping("/persorario")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<PersOrario> updatePersOrario(@Valid @RequestBody PersOrario persOrarioDTO) {
		logger.debug("REST request to update PersOrario : {}", persOrarioDTO);
		Optional<PersOrario> updatedLeaves = persOrarioService.updatePersOrario(persOrarioDTO);

		return ResponseUtil.wrapOrNotFound(updatedLeaves, HeaderUtil.createAlert(applicationName,
				"A pers orario is updated with identifier " + persOrarioDTO.getId(), persOrarioDTO.getCf()));
	}

	/**
	 * {@code GET /persorario/:id} : get the "id" of pers orario.
	 *
	 * @param id the id of the pers orario to find.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the pers orario, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/persorario/{id:" + Constants.VALID_NUMBER_REGEX + "}")
	public ResponseEntity<PersOrario> getPersOrario(@PathVariable String id) {
		logger.debug("REST request to get Pers Orario : {}", id);
		return ResponseUtil.wrapOrNotFound(persOrarioService.getPersOrarioById(Integer.valueOf(id)));

	}

	/**
	 * {@code DELETE /persorario/:id} : delete the "id" Leave.
	 *
	 * @param id the id of the pers orario to delete.
	 * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
	 */
	@DeleteMapping("/persorario/{id:" + Constants.VALID_NUMBER_REGEX + "}")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<Void> deletePersOrario(@PathVariable String id) {
		logger.debug("REST request to delete Pers Orario id: {}", id);
		persOrarioService.deletePersOrario(Integer.valueOf(id));
		return ResponseEntity.noContent()
				.headers(HeaderUtil.createAlert(applicationName, "A Pers Orario is deleted with identifier " + id, id))
				.build();
	}

}
