package it.cnr.isti.epasmed.web.rest;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;

import io.github.jhipster.web.util.HeaderUtil;
import io.github.jhipster.web.util.PaginationUtil;
import io.github.jhipster.web.util.ResponseUtil;
import it.cnr.isti.epasmed.config.Constants;
import it.cnr.isti.epasmed.domain.TabsSI;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;
import it.cnr.isti.epasmed.service.TabsSIService;
import it.cnr.isti.epasmed.web.rest.errors.BadRequestAlertException;

/**
 * REST controller for managing Table SI.
 * <p>
 * This class accesses the {@link TabsSI} entity.
 * 
 * 
 */
@RestController
@RequestMapping("/api")
public class TabsSIResource {
	private static final List<String> ALLOWED_ORDERED_PROPERTIES = Collections
			.unmodifiableList(Arrays.asList("id", "nome", "operazioni", "idFlusso", "lastUpdate"));

	private final Logger log = LoggerFactory.getLogger(TabsSIResource.class);

	@Value("${jhipster.clientApp.name}")
	private String applicationName;

	private final TabsSIService tabsSIService;

	public TabsSIResource(TabsSIService tabsSIService) {
		this.tabsSIService = tabsSIService;

	}

	/**
	 * {@code GET /tabssi} : get all tabs of sistema informativo.
	 *
	 * @param pageable the pagination information.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         all tabs of sistema informativo.
	 */
	@GetMapping("/tabssi")
	public ResponseEntity<List<TabsSI>> getAllTabsSI(Pageable pageable) {
		log.info("REST request to read Table SI : {}", pageable);

		if (!onlyContainsAllowedProperties(pageable)) {
			return ResponseEntity.badRequest().build();
		}

		final Page<TabsSI> page = tabsSIService.getAllTabsSI(pageable);
		HttpHeaders headers = PaginationUtil
				.generatePaginationHttpHeaders(ServletUriComponentsBuilder.fromCurrentRequest(), page);
		return new ResponseEntity<>(page.getContent(), headers, HttpStatus.OK);
	}

	private boolean onlyContainsAllowedProperties(Pageable pageable) {
		return pageable.getSort().stream().map(Sort.Order::getProperty).allMatch(ALLOWED_ORDERED_PROPERTIES::contains);
	}

	/**
	 * {@code POST  /tabssi} : Creates a new table for SI.
	 * <p>
	 *
	 * @param tabsSIDTO table to create.
	 * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with
	 *         body the new table SI, or with status {@code 400 (Bad Request)} if
	 *         the id is already in use.
	 * @throws URISyntaxException if the Location URI syntax is incorrect.
	 */
	@PostMapping("/tabssi")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<TabsSI> createTabsSI(@Valid @RequestBody TabsSI tabsSIDTO) throws URISyntaxException {
		log.info("REST request to save Table SI : {}", tabsSIDTO);

		if (tabsSIDTO.getId() != null) {
			throw new BadRequestAlertException("A new table SI cannot already have an ID", "TabsSI", "idexists");
		} else {
			TabsSI newTabsSI = tabsSIService.createTabsSI(tabsSIDTO);
			return ResponseEntity.created(new URI("/api/tabssi/" + newTabsSI.getId()))
					.headers(HeaderUtil.createAlert(applicationName,
							"A user is created with identifier " + newTabsSI.getId(), newTabsSI.getNome()))
					.body(newTabsSI);
		}
	}

	/**
	 * {@code PUT /tabssi} : Updates an tab SI.
	 *
	 * @param tabsSIDTO the table of SI to update.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the updated table of IS.
	 * 
	 */
	@PutMapping("/tabssi")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<TabsSI> updateTabsSI(@Valid @RequestBody TabsSI tabsSIDTO) {
		log.debug("REST request to update Table SI : {}", tabsSIDTO);
		Optional<TabsSI> updatedTabsSI = tabsSIService.updateTabsSI(tabsSIDTO);

		return ResponseUtil.wrapOrNotFound(updatedTabsSI, HeaderUtil.createAlert(applicationName,
				"A table SI is updated with identifier " + tabsSIDTO.getId(), tabsSIDTO.getNome()));
	}

	/**
	 * {@code GET /tabssi/:id} : get the "id" of table SI.
	 *
	 * @param id the id of the table SI to find.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the tab SI, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/tabssi/{id:" + Constants.VALID_NUMBER_REGEX + "}")
	public ResponseEntity<TabsSI> getTabsSI(@PathVariable String id) {
		log.debug("REST request to get Table SI : {}", id);
		return ResponseUtil.wrapOrNotFound(tabsSIService.getTabsSIById(Long.valueOf(id)));

	}

	/**
	 * {@code DELETE /tabssi/:id} : delete the "id" TabsSI.
	 *
	 * @param id the id of the table SI to delete.
	 * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
	 */
	@DeleteMapping("/tabssi/{id:" + Constants.VALID_NUMBER_REGEX + "}")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<Void> deleteTabsSI(@PathVariable String id) {
		log.debug("REST request to delete Table SI id: {}", id);
		tabsSIService.deleteTabsSI(Long.valueOf(id));
		return ResponseEntity.noContent()
				.headers(HeaderUtil.createAlert(applicationName, "A Table IS is deleted with identifier " + id, id))
				.build();
	}

}
