package it.cnr.isti.epasmed.web.rest;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;

import io.github.jhipster.web.util.HeaderUtil;
import io.github.jhipster.web.util.PaginationUtil;
import io.github.jhipster.web.util.ResponseUtil;
import it.cnr.isti.epasmed.config.Constants;
import it.cnr.isti.epasmed.domain.TimeCardsReporting;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;
import it.cnr.isti.epasmed.service.TimeCardsReportingService;
import it.cnr.isti.epasmed.web.rest.errors.BadRequestAlertException;

/**
 * REST controller for managing TimeCardsReporting.
 * <p>
 * This class accesses the {@link TimeCardsReporting} entity.
 * 
 * 
 */
@RestController
@RequestMapping("/api")
public class TimeCardsReportingResource {
	private static final List<String> ALLOWED_ORDERED_PROPERTIES = Collections
			.unmodifiableList(Arrays.asList("id", "year", "month", "idFlusso", "lastUpdate"));

	private final Logger log = LoggerFactory.getLogger(TimeCardsReportingResource.class);

	@Value("${jhipster.clientApp.name}")
	private String applicationName;

	private final TimeCardsReportingService timeCardsReportingService;

	public TimeCardsReportingResource(TimeCardsReportingService timeCardsReportingService) {
		this.timeCardsReportingService = timeCardsReportingService;

	}

	/**
	 * {@code GET /timecardsreporting} : get all timecards reporting.
	 *
	 * @param pageable the pagination information.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         all TimeCardsReporting.
	 */
	@GetMapping("/timecardsreporting")
	public ResponseEntity<List<TimeCardsReporting>> getAllTimeCardsReporting(Pageable pageable) {
		log.info("REST request to read all TimeCardsReporting : {}", pageable);

		if (!onlyContainsAllowedProperties(pageable)) {
			return ResponseEntity.badRequest().build();
		}

		final Page<TimeCardsReporting> page = timeCardsReportingService.getAllTimeCardsReporting(pageable);
		HttpHeaders headers = PaginationUtil
				.generatePaginationHttpHeaders(ServletUriComponentsBuilder.fromCurrentRequest(), page);
		return new ResponseEntity<>(page.getContent(), headers, HttpStatus.OK);
	}

	private boolean onlyContainsAllowedProperties(Pageable pageable) {
		return pageable.getSort().stream().map(Sort.Order::getProperty).allMatch(ALLOWED_ORDERED_PROPERTIES::contains);
	}

	/**
	 * {@code POST  /timecardsreporting} : Creates a new timecards reporting.
	 * <p>
	 * 
	 * @param timeCardsReporting the timecards reporting to create.
	 * @return TimeCardsReporting the new time cards reporting.
	 * @throws URISyntaxException if the Location URI syntax is incorrect.
	 * 
	 */
	@PostMapping("/timecardsreporting")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<TimeCardsReporting> createTimeCardsReporting(
			@Valid @RequestBody TimeCardsReporting timeCardsReporting) throws URISyntaxException {
		log.info("REST request to save TimeCardsReporting : {}", timeCardsReporting);

		if (timeCardsReporting.getId() != null) {
			throw new BadRequestAlertException("A new timecards reporting already have an ID", "TimeCardsReporting",
					"idexists");
		} else {
			TimeCardsReporting newTcR = timeCardsReportingService.createTimeCardsReporting(timeCardsReporting);
			StringBuilder m = new StringBuilder();
			m.append(newTcR.getYear());
			m.append("/");
			m.append(newTcR.getMonth());

			return ResponseEntity.created(new URI("/api/timecardsreporting/" + newTcR.getId()))
					.headers(HeaderUtil.createAlert(applicationName,
							"A timecards reporting is created with identifier " + newTcR.getId(), m.toString()))
					.body(newTcR);
		}
	}

	/**
	 * {@code PUT /timecardsreporting} : Updates an existing timecards reporting.
	 *
	 * @param timeCardsReportingDTO the timecards reporting to update.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the updated timecards reporting.
	 *
	 */
	@PutMapping("/timecardsreporting")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<TimeCardsReporting> updateTimeCardsReporting(
			@Valid @RequestBody TimeCardsReporting timeCardsReportingDTO) {
		log.debug("REST request to update TimeCards Reporting : {}", timeCardsReportingDTO);
		Optional<TimeCardsReporting> updatedTimeCardsReporting = timeCardsReportingService
				.updateTimeCardsReporting(timeCardsReportingDTO);
		StringBuilder m = new StringBuilder();
		if (updatedTimeCardsReporting.isPresent()) {
			m.append(updatedTimeCardsReporting.get().getYear());
			m.append("/");
			m.append(updatedTimeCardsReporting.get().getMonth());
		} else {

		}

		return ResponseUtil.wrapOrNotFound(updatedTimeCardsReporting, HeaderUtil.createAlert(applicationName,
				"A timecards reporting is updated with identifier " + timeCardsReportingDTO.getId(), m.toString()));
	}

	/**
	 * {@code GET /timecardsreporting/:id} : get timecards reporting by id.
	 *
	 * @param id the id of the timecards reporting to find.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the timecards reporting, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/timecardsreporting/{id:" + Constants.VALID_NUMBER_REGEX + "}")
	public ResponseEntity<TimeCardsReporting> getTimeCardsReporting(@PathVariable String id) {
		log.debug("REST request to get TimeCards Reporting : {}", id);
		return ResponseUtil.wrapOrNotFound(timeCardsReportingService.getTimeCardsReportingById(Long.valueOf(id)));
	}

	/**
	 * {@code DELETE /timecardsreporting/:id} : delete the timecards reporting by
	 * id.
	 *
	 * @param id the id of the timecards reporting to delete.
	 * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
	 */
	@DeleteMapping("/timecardsreporting/{id:" + Constants.VALID_NUMBER_REGEX + "}")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<Void> deleteUser(@PathVariable String id) {
		log.debug("REST request to delete TimeCards Reporting id: {}", id);
		timeCardsReportingService.deleteTimeCardsReporting(Long.valueOf(id));
		return ResponseEntity.noContent().headers(
				HeaderUtil.createAlert(applicationName, "A timecards reporting is deleted with identifier " + id, id))
				.build();
	}
	
	
	/**
	 * {@code GET /timecardsreporting/last} : get last timecards reporting.
	 *
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the timecards reporting, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/timecardsreporting/last")
	public ResponseEntity<TimeCardsReporting> getTimeCardsReporting() {
		log.debug("REST request to get Last TimeCards Reporting");
		return ResponseUtil.wrapOrNotFound(timeCardsReportingService.getLastTimeCardsReporting());
	}


}
