package it.cnr.isti.epasmed.web.rest.epas;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Optional;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import io.github.jhipster.web.util.HeaderUtil;
import io.github.jhipster.web.util.ResponseUtil;
import it.cnr.isti.epasmed.epas.dto.EPASAffiliationsDTO;
import it.cnr.isti.epasmed.epas.model.EPASAffiliations;
import it.cnr.isti.epasmed.epas.service.EPASAffiliationsService;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;
import it.cnr.isti.epasmed.web.rest.errors.BadRequestAlertException;

@RestController
@RequestMapping("/api/epas")
public class EPASAffiliationsResource {

	private final Logger logger = LoggerFactory.getLogger(getClass());

	@Value("${jhipster.clientApp.name}")
	private String applicationName;

	private final EPASAffiliationsService epasAffiliationsService;

	public EPASAffiliationsResource(EPASAffiliationsService epasAffiliationsServiceService) {
		this.epasAffiliationsService = epasAffiliationsServiceService;

	}

	/**
	 * {@code GET /affiliations/:id} : get affiliations by id.
	 *
	 * @param id the id of the affiliations to find.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the EPAS Affiliations, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/affiliations/{id}")
	public ResponseEntity<EPASAffiliations> getEPASAffiliationsById(@PathVariable String id) {
		logger.info("REST request to get ePAS Affiliations by Id: {}", id);
		EPASAffiliations epasAffiliations = epasAffiliationsService.getById(id);
		logger.info("Retrieved Affiliations: {}", epasAffiliations);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasAffiliations));

	}

	/**
	 * {@code GET /affiliations/byGroup} : get the affiliations list by group.
	 *
	 * @param id              the group id.
	 * @param includeInactive true if include inactive affiliations.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the list of EPAS Affiliations, or with status
	 *         {@code 404 (Not Found)}.
	 */
	@GetMapping("/affiliations/byGroup")
	public ResponseEntity<List<EPASAffiliations>> getEPASAffiliationsByGroup(@RequestParam("id") String id,
			@RequestParam("includeInactive") Optional<Boolean> includeInactive) {
		logger.info("REST request to get ePAS Affiliations list by Group Id: {}", id);
		boolean includeInctv=false;
		if (includeInactive.isPresent()) {
			logger.info("Include Inactive: {}", includeInactive.get());
			includeInctv=includeInactive.get();
		} 
		List<EPASAffiliations> epasAffiliationsList = epasAffiliationsService.getByGroup(id, includeInctv);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasAffiliationsList));
	}

	/**
	 * {@code GET /affiliations/byPerson} : get the affiliations list by person.
	 *
	 * @param id         the person id.
	 * @param fiscalCode the person fiscal code.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the list of EPAS Affiliations, or with status
	 *         {@code 404 (Not Found)}.
	 */
	@GetMapping("/affiliations/byPerson")
	public ResponseEntity<List<EPASAffiliations>> getEPASAffiliationsByPerson(@RequestParam("id") Optional<String> id,
			@RequestParam("fiscalCode") Optional<String> fiscalCode) {
		List<EPASAffiliations> epasAffiliationsList = null;

		if (id.isPresent()) {
			logger.info("REST request to get ePAS Affiliations by Person id: {}", id.get());
			epasAffiliationsList = epasAffiliationsService.getByPersonId(id.get());
		} else {
			if (fiscalCode.isPresent()) {
				logger.info("REST request to get ePAS Affiliations by Person fiscal code: {}", fiscalCode.get());
				epasAffiliationsList = epasAffiliationsService.getByPersonFiscalcode(fiscalCode.get());
			}
		}
		return ResponseUtil.wrapOrNotFound(Optional.of(epasAffiliationsList));
	}

	/**
	 * {@code POST  /affiliations} : Creates a new affiliations.
	 * <p>
	 * Creates a new affiliations if the id is not already used.
	 *
	 * @param epasAffiliationsDTO the affiliation to create.
	 * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with
	 *         body the new ePAS Affiliation, or with status
	 *         {@code 400 (Bad Request)} if the id is already in use.
	 * @throws URISyntaxException       if the Location URI syntax is incorrect.
	 * @throws BadRequestAlertException {@code 400 (Bad Request)} if the id is
	 *                                  already in use.
	 */
	@PostMapping("/affiliations")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<EPASAffiliations> createEPASAffiliations(
			@Valid @RequestBody EPASAffiliationsDTO epasAffiliationsDTO) throws URISyntaxException {
		logger.debug("REST request to create EPAS Affiliation: {}", epasAffiliationsDTO);

		if (epasAffiliationsDTO.getId() != null) {
			throw new BadRequestAlertException("A new ePAS Affiliation already have an ID", "EPASAffiliations",
					"idexists");
		} else {
			EPASAffiliations createdEPASAffiliations = epasAffiliationsService.create(epasAffiliationsDTO);
			return ResponseEntity.created(new URI("/api/epas/affiliations/" + createdEPASAffiliations.getId()))
					.headers(HeaderUtil.createAlert(applicationName,
							"A ePAS Affiliations is created with identifier " + createdEPASAffiliations.getId(),
							createdEPASAffiliations.getId()))
					.body(createdEPASAffiliations);
		}
	}

	/**
	 * {@code PUT /affiliations/:id} : Updates an existing ePAS Affiliation by id.
	 *
	 * @param id                  the Affiliation id
	 * @param epasAffiliationsDTO the new ePAS Affiliation info.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the updated ePAS Affiliations.
	 * 
	 */
	@PutMapping("/affiliations/{id}")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<EPASAffiliations> updateEPASAffiliationsById(@PathVariable String id,
			@Valid @RequestBody EPASAffiliationsDTO epasAffiliationsDTO) {
		logger.debug("REST request to update ePAS Affiliations : {} by {}", id, epasAffiliationsDTO);
		epasAffiliationsService.updateById(id, epasAffiliationsDTO);
		EPASAffiliations updatedEPASAffiliations = epasAffiliationsService.getById(id);
		Optional<EPASAffiliations> oEPASAffiliations = Optional.of(updatedEPASAffiliations);

		return ResponseUtil.wrapOrNotFound(oEPASAffiliations,
				HeaderUtil.createAlert(applicationName,
						"A ePAS Affiliation is updated with identifier " + updatedEPASAffiliations.getId(),
						updatedEPASAffiliations.getId()));
	}

	/**
	 * {@code DELETE /affiliations/:id} : delete ePAS Affiliation by id.
	 *
	 * @param id the id of the ePAS Affiliation to delete.
	 * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
	 */
	@DeleteMapping("/affiliations/{id}")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<Void> deleteEPASAffiliationById(@PathVariable String id) {
		logger.debug("REST request to delete ePAS Affiliation by id: {}", id);
		epasAffiliationsService.deleteById(id);
		return ResponseEntity.noContent()
				.headers(HeaderUtil.createAlert(applicationName, "A Affiliation is deleted with identifier " + id, id))
				.build();
	}

}
