package it.cnr.isti.epasmed.web.rest.epas;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Optional;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import io.github.jhipster.web.util.HeaderUtil;
import io.github.jhipster.web.util.ResponseUtil;
import it.cnr.isti.epasmed.epas.dto.EPASChildrenDTO;
import it.cnr.isti.epasmed.epas.model.EPASChildren;
import it.cnr.isti.epasmed.epas.service.EPASChildrenService;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;
import it.cnr.isti.epasmed.web.rest.errors.BadRequestAlertException;

@RestController
@RequestMapping("/api/epas")
public class EPASChildrenResource {

	private final Logger logger = LoggerFactory.getLogger(getClass());

	@Value("${jhipster.clientApp.name}")
	private String applicationName;

	private final EPASChildrenService epasChildrenService;

	public EPASChildrenResource(EPASChildrenService epasChildrenServiceService) {
		this.epasChildrenService = epasChildrenServiceService;

	}

	/**
	 * {@code GET /child/:id} : get children by id.
	 *
	 * @param id the id of the children to find.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the EPAS Children, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/child/{id}")
	public ResponseEntity<EPASChildren> getEPASChildrenById(@PathVariable String id) {
		logger.info("REST request to get ePAS Children by Id: {}", id);
		EPASChildren epasChildren = epasChildrenService.getById(id);
		logger.info("Retrieved Children: {}", epasChildren);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasChildren));

	}

	/**
	 * {@code GET /child/byPerson} : get children by person
	 *
	 * @param id the id of the person.
	 * @param fiscalCode the fiscal code of the person.
	 * @param email the email of the person.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the EPAS Children, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/child/byPerson")
	public ResponseEntity<List<EPASChildren>> getEPASChildrenByPerson(@RequestParam("id") Optional<String> id,
			@RequestParam("fiscalCode") Optional<String> fiscalCode,
			@RequestParam("email") Optional<String> email) {
		
		
		List<EPASChildren> listEpasChildren = null;
		if (id.isPresent()) {
			logger.info("REST request to get ePAS Children by Person id: {}", id.get());
			listEpasChildren = epasChildrenService.getByPersonId(id.get());
		} else {
			if (fiscalCode.isPresent()) {
				logger.info("REST request to get ePAS Children by Person fiscalcode: {}", fiscalCode.get());
				listEpasChildren = epasChildrenService.getByPersonFiscalCode(fiscalCode.get());
			} else {
				if (email.isPresent()) {
					logger.info("REST request to get ePAS Children by Person email: {}", email.get());
					listEpasChildren = epasChildrenService.getByPersonEmail(email.get());
				} else {
					return ResponseUtil.wrapOrNotFound(Optional.of(listEpasChildren), HeaderUtil.createFailureAlert(applicationName,false,
							"","","Invalid parameter in call"));
				
				}
			}
		}
		logger.info("Retrieved Certification: {}", listEpasChildren);
		return ResponseUtil.wrapOrNotFound(Optional.of(listEpasChildren));
	}



	/**
	 * {@code POST  /child} : Creates a new child.
	 * <p>
	 *
	 * @param epasChildrenDTO the user to create.
	 * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with
	 *         body the new ePAS Child, or with status {@code 400 (Bad Request)}
	 * @throws URISyntaxException       if the Location URI syntax is incorrect.
	 * @throws BadRequestAlertException {@code 400 (Bad Request)} if the id is
	 *                                  already in use.
	 */
	@PostMapping("/child")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<EPASChildren> createEPASChildren(@Valid @RequestBody EPASChildrenDTO epasChildrenDTO)
			throws URISyntaxException {
		logger.debug("REST request to create EPAS Child: {}", epasChildrenDTO);

		EPASChildren createdEPASChildren = epasChildrenService.create(epasChildrenDTO);
		StringBuilder sb = new StringBuilder();
		sb.append(createdEPASChildren.getSurname());
		sb.append(" ");
		sb.append(createdEPASChildren.getName());
		
		
		return ResponseEntity.created(new URI("/api/epas/child/" + createdEPASChildren.getId()))
				.headers(HeaderUtil.createAlert(applicationName,
						"A ePAS Child is created with identifier " + createdEPASChildren.getId(),
						sb.toString()))
				.body(createdEPASChildren);

	}

	/**
	 * {@code PUT /child/:id} : Updates an existing ePAS Children by id.
	 *
	 * @param epasChildrenDTO the new ePAS Children info.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the updated ePAS Children.
	 * 
	 */
	@PutMapping("/child/{id}")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<EPASChildren> updateEPASChildrenById(@PathVariable String id,
			@Valid @RequestBody EPASChildrenDTO epasChildrenDTO) {
		logger.debug("REST request to update ePAS Children : {} by {}", id, epasChildrenDTO);
		epasChildrenService.updateById(id, epasChildrenDTO);
		EPASChildren updatedEPASChildren = epasChildrenService.getById(id);
		Optional<EPASChildren> oEPASChildren = Optional.of(updatedEPASChildren);
		StringBuilder sb = new StringBuilder();
		sb.append(updatedEPASChildren.getSurname());
		sb.append(" ");
		sb.append(updatedEPASChildren.getName());

		return ResponseUtil.wrapOrNotFound(oEPASChildren, HeaderUtil.createAlert(applicationName,
				"A ePAS Child is updated with identifier " + updatedEPASChildren.getId(), sb.toString()));
	}

	/**
	 * {@code DELETE /child/:id} : delete ePAS Children by id.
	 *
	 * @param id the id of the ePAS Children to delete.
	 * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
	 */
	@DeleteMapping("/child/{id}")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<Void> deleteEPASChildrenById(@PathVariable String id) {
		logger.debug("REST request to delete ePAS Children by id: {}", id);
		epasChildrenService.deleteById(id);
		return ResponseEntity.noContent()
				.headers(HeaderUtil.createAlert(applicationName, "A Child is deleted with identifier " + id, id))
				.build();
	}

}
