package it.cnr.isti.epasmed.web.rest.epas;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Optional;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import io.github.jhipster.web.util.HeaderUtil;
import io.github.jhipster.web.util.ResponseUtil;
import it.cnr.isti.epasmed.epas.dto.EPASContractsDTO;
import it.cnr.isti.epasmed.epas.model.EPASContracts;
import it.cnr.isti.epasmed.epas.service.EPASContractsService;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;
import it.cnr.isti.epasmed.web.rest.errors.BadRequestAlertException;

@RestController
@RequestMapping("/api/epas")
public class EPASContractsResource {

	private final Logger logger = LoggerFactory.getLogger(getClass());

	@Value("${jhipster.clientApp.name}")
	private String applicationName;

	private final EPASContractsService epasContractsService;

	public EPASContractsResource(EPASContractsService epasContractsServiceService) {
		this.epasContractsService = epasContractsServiceService;

	}

	/**
	 * {@code GET /contracts/:id} : get contracts by id.
	 *
	 * @param id the id of the contracts to find.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the EPAS Contracts, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/contracts/{id}")
	public ResponseEntity<EPASContracts> getEPASContractsById(@PathVariable String id) {
		logger.info("REST request to get ePAS Contracts by Id: {}", id);
		EPASContracts epasContracts = epasContractsService.getById(id);
		logger.info("Retrieved Contracts: {}", epasContracts);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasContracts));

	}
	
	/**
	 * {@code GET /contracts/map} : get contracts map.
	 *
	 * @param officeId the office Id.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the EPAS Contracts Map, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/contracts/map")
	public ResponseEntity<LinkedHashMap<String,EPASContracts>> getEPASContractsMap(@RequestParam("officeId") String officeId) {
		logger.info("REST request to get ePAS Contracts Map by officeId: {}", officeId);
		LinkedHashMap<String,EPASContracts> epasContractsMap = epasContractsService.getMap(officeId);
		logger.info("Retrieved Contracts Map: {}", epasContractsMap);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasContractsMap));

	}

	

	/**
	 * {@code GET /contracts/byPerson} : get contracts by Person 
	 *
	 * @param id the id of the person.
	 * @param fiscalCode the fiscal code of the person.
	 * @param email the email of the person.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the EPAS Contracts, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/contracts/byPerson")
	public ResponseEntity<List<EPASContracts>> getEPASContractsByPerson(@RequestParam("id") Optional<String> id,
			@RequestParam("fiscalCode") Optional<String> fiscalCode,
			@RequestParam("email") Optional<String> email) {
		List<EPASContracts> epasContractsList = null;
		if (id.isPresent()) {
			logger.info("REST request to get ePAS Contracts by Person: {}", id.get());
			epasContractsList = epasContractsService.getByPersonId(id.get());
		} else {
			if (fiscalCode.isPresent()) {
				logger.info("REST request to get ePAS Contracts by Person fiscal code: {}", fiscalCode.get());
				epasContractsList = epasContractsService.getByPersonFiscalcode(fiscalCode.get());
			} else {
				if (email.isPresent()) {
					logger.info("REST request to get ePAS Contracts by Person email: {}", email.get());
					epasContractsList = epasContractsService.getByPersonEmail(email.get());
				} else {
					return ResponseUtil.wrapOrNotFound(Optional.of(epasContractsList), HeaderUtil.createFailureAlert(applicationName,false,
							"","","Invalid parameter in call"));
		
				}
			}
		}
		logger.info("Retrieved List of Contracts: {}", epasContractsList);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasContractsList));
	}


	/**
	 * {@code POST  /contracts} : Creates a new contracts.
	 * <p>
	 * Creates a new contracts if the id is not already used.
	 *
	 * @param epasContractsDTO the user to create.
	 * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with
	 *         body the new ePAS Contract, or with status {@code 400 (Bad Request)}
	 *         if the id is already in use.
	 * @throws URISyntaxException       if the Location URI syntax is incorrect.
	 * @throws BadRequestAlertException {@code 400 (Bad Request)} if the id is
	 *                                  already in use.
	 */
	@PostMapping("/contracts")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<EPASContracts> createEPASContracts(@Valid @RequestBody EPASContractsDTO epasContractsDTO)
			throws URISyntaxException {
		logger.debug("REST request to create EPAS Contract: {}", epasContractsDTO);

		if (epasContractsDTO.getId() != null) {
			throw new BadRequestAlertException("A new ePAS Contract already have an ID", "EPASContracts", "idexists");
		} else {
			EPASContracts createdEPASContracts = epasContractsService.create(epasContractsDTO);
			return ResponseEntity.created(new URI("/api/epas/contracts/" + createdEPASContracts.getId()))
					.headers(HeaderUtil.createAlert(applicationName,
							"A ePAS Contracts is created with identifier " + createdEPASContracts.getId(),
							createdEPASContracts.getPerson().getFullname()))
					.body(createdEPASContracts);
		}
	}

	/**
	 * {@code PUT /contracts/:id} : Updates an existing ePAS Contract by id.
	 *
	 * @param epasContracts the new ePAS Contract info.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the updated ePAS Contracts.
	 * 
	 */
	@PutMapping("/contracts/{id}")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<EPASContracts> updateEPASContractsById(@PathVariable String id,
			@Valid @RequestBody EPASContractsDTO epasContractsDTO) {
		logger.debug("REST request to update ePAS Contracts : {} by {}", id, epasContractsDTO);
		epasContractsService.updateById(id, epasContractsDTO);
		EPASContracts updatedEPASContracts = epasContractsService.getById(id);
		Optional<EPASContracts> oEPASContracts = Optional.of(updatedEPASContracts);

		return ResponseUtil.wrapOrNotFound(oEPASContracts,
				HeaderUtil.createAlert(applicationName,
						"A ePAS Contract is updated with identifier " + updatedEPASContracts.getId(),
						updatedEPASContracts.getPerson().getFullname()));
	}

	/**
	 * {@code DELETE /contracts/setPreviousContract} : set ePAS Previous Contract.
	 *
	 * @param id the id of the ePAS Contract to set.
	 * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
	 */
	@DeleteMapping("/contracts/setPreviousContract")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<Void> setEPASContractPrevious(@RequestParam("id") String id) {
		logger.debug("REST request Set Previous Contract ePAS by id: {}", id);
		epasContractsService.setPreviousContract(id);
		return ResponseEntity.noContent().headers(HeaderUtil.createAlert(applicationName,
				"Setted to previous the EPAS Contract with identifier " + id, id)).build();
	}

	/**
	 * {@code DELETE /contracts/unsetPreviousContract} : unset ePAS Previous
	 * Contract.
	 *
	 * @param id the id of the ePAS Contract to unset.
	 * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
	 */
	@DeleteMapping("/contracts/unsetPreviousContract")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<Void> setEPASContractUnsetPrevious(@RequestParam("id") String id) {
		logger.debug("REST request Unset Previous Contract ePAS by id: {}", id);
		epasContractsService.setUnsetPreviousContract(id);
		return ResponseEntity.noContent().headers(HeaderUtil.createAlert(applicationName,
				"Unsetted to previous the EPAS Contract with identifier " + id, id)).build();
	}

	/**
	 * {@code DELETE /contracts/:id} : delete ePAS Contract by id.
	 *
	 * @param id the id of the ePAS Contract to delete.
	 * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
	 */
	@DeleteMapping("/contracts/{id}")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<Void> deleteEPASContractById(@PathVariable String id) {
		logger.debug("REST request to delete ePAS Contract by id: {}", id);
		epasContractsService.deleteById(id);
		return ResponseEntity.noContent()
				.headers(HeaderUtil.createAlert(applicationName, "A Contract is deleted with identifier " + id, id))
				.build();
	}

}
