package it.cnr.isti.epasmed.web.rest.epas;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Optional;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import io.github.jhipster.web.util.HeaderUtil;
import io.github.jhipster.web.util.ResponseUtil;
import it.cnr.isti.epasmed.epas.dto.EPASGroupsDTO;
import it.cnr.isti.epasmed.epas.model.EPASGroups;
import it.cnr.isti.epasmed.epas.service.EPASGroupsService;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;
import it.cnr.isti.epasmed.web.rest.errors.BadRequestAlertException;

@RestController
@RequestMapping("/api/epas")
public class EPASGroupsResource {

	private final Logger logger = LoggerFactory.getLogger(getClass());

	@Value("${jhipster.clientApp.name}")
	private String applicationName;

	private final EPASGroupsService epasGroupsService;

	public EPASGroupsResource(EPASGroupsService epasGroupsServiceService) {
		this.epasGroupsService = epasGroupsServiceService;

	}

	/**
	 * {@code GET /groups/:id} : get groups by id.
	 *
	 * @param id the id of the groups to find.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the EPAS Groups, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/groups/{id}")
	public ResponseEntity<EPASGroups> getEPASGroupsById(@PathVariable String id) {
		logger.info("REST request to get ePAS Groups by Id: {}", id);
		EPASGroups epasGroups = epasGroupsService.getById(id);
		logger.info("Retrieved Groups: {}", epasGroups);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasGroups));

	}

	

	/**
	 * {@code GET /groups} : get the groups list.
	 *
	 * @param officeId the office id.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the list of EPAS Groups, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/groups")
	public ResponseEntity<List<EPASGroups>> getEPASGroups(@RequestParam("officeId") String id) {
		logger.info("REST request to get ePAS Groups list for Office: {}",id);
		List<EPASGroups> epasGroupsList = epasGroupsService.getList(id);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasGroupsList));

	}

	/**
	 * {@code POST  /groups} : Creates a new groups.
	 * <p>
	 * Creates a new groups if the id is not already used.
	 *
	 * @param epasGroupsDTO the user to create.
	 * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with
	 *         body the new ePAS Group, or with status {@code 400 (Bad Request)} if
	 *         the id is already in use.
	 * @throws URISyntaxException       if the Location URI syntax is incorrect.
	 * @throws BadRequestAlertException {@code 400 (Bad Request)} if the id is
	 *                                  already in use.
	 */
	@PostMapping("/groups")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<EPASGroups> createEPASGroups(@Valid @RequestBody EPASGroupsDTO epasGroupsDTO)
			throws URISyntaxException {
		logger.debug("REST request to create EPAS Group: {}", epasGroupsDTO);

		if (epasGroupsDTO.getId() != null) {
			throw new BadRequestAlertException("A new ePAS Group already have an ID", "EPASGroups", "idexists");
		} else {
			EPASGroups createdEPASGroups = epasGroupsService.create(epasGroupsDTO);
			return ResponseEntity.created(new URI("/api/epas/groups/" + createdEPASGroups.getId()))
					.headers(HeaderUtil.createAlert(applicationName,
							"A ePAS Groups is created with identifier " + createdEPASGroups.getId(),
							createdEPASGroups.getName()))
					.body(createdEPASGroups);
		}
	}

	/**
	 * {@code PUT /groups/:id} : Updates an existing ePAS Group by id.
	 *
	 * @param epasGroups the new ePAS Group info.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the updated ePAS Groups.
	 * 
	 */
	@PutMapping("/groups/{id}")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<EPASGroups> updateEPASGroupsById(@PathVariable String id,
			@Valid @RequestBody EPASGroupsDTO epasGroupsDTO) {
		logger.debug("REST request to update ePAS Groups : {} by {}", id, epasGroupsDTO);
		epasGroupsService.updateById(id,epasGroupsDTO);
		EPASGroups updatedEPASGroups=epasGroupsService.getById(id);
		Optional<EPASGroups> oEPASGroups =  Optional.of(updatedEPASGroups);

		return ResponseUtil.wrapOrNotFound(oEPASGroups, HeaderUtil.createAlert(applicationName,
				"A ePAS Group is updated with identifier " + updatedEPASGroups.getId(), updatedEPASGroups.getName()));
	}

	
	
	

	/**
	 * {@code DELETE /groups/:id} : delete ePAS Group by id.
	 *
	 * @param id the id of the ePAS Group to delete.
	 * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
	 */
	@DeleteMapping("/groups/{id}")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<Void> deleteEPASGroupById(@PathVariable String id) {
		logger.debug("REST request to delete ePAS Group by id: {}", id);
		epasGroupsService.deleteById(id);
		return ResponseEntity.noContent()
				.headers(HeaderUtil.createAlert(applicationName, "A Group is deleted with identifier " + id, id))
				.build();
	}
	

	
	

}
