package it.cnr.isti.epasmed.web.rest.epas;

import java.util.List;
import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import io.github.jhipster.web.util.HeaderUtil;
import io.github.jhipster.web.util.ResponseUtil;
import it.cnr.isti.epasmed.epas.model.EPASLeaves;
import it.cnr.isti.epasmed.epas.service.EPASLeavesService;

@RestController
@RequestMapping("/api/epas")
public class EPASLeavesResource {

	private final Logger logger = LoggerFactory.getLogger(getClass());

	@Value("${jhipster.clientApp.name}")
	private String applicationName;

	private final EPASLeavesService epasLeavesService;

	public EPASLeavesResource(EPASLeavesService epasLeavesService) {
		this.epasLeavesService = epasLeavesService;

	}

	/**
	 * {@code GET /leaves/byPersonAndYear} : get contracts by person and year
	 *
	 * @param id             the id of the person.
	 * @param fiscalCode     the fiscal code of the person.
	 * @param email          the email of the person.
	 * @param includeDetails a true or false flag that specifies whether or not to
	 *                       include absence details.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the EPAS Leaves, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/leaves/byPersonAndYear")
	public ResponseEntity<List<EPASLeaves>> getEPASContractsByPerson(@RequestParam("id") Optional<String> id,
			@RequestParam("fiscalCode") Optional<String> fiscalCode, @RequestParam("email") Optional<String> email,
			@RequestParam("year") String year, @RequestParam("includeDetails") String includeDetails) {
		List<EPASLeaves> epasLeavesList = null;
		if (id.isPresent()) {
			logger.info("REST request to get ePAS Leaves by Person: {}", id.get());
			epasLeavesList = epasLeavesService.getLeavesByPersonId(id.get(), year, includeDetails);
		} else {
			if (fiscalCode.isPresent()) {
				logger.info("REST request to get ePAS Leaves by Person fiscal code: {}", fiscalCode.get());
				epasLeavesList = epasLeavesService.getLeavesByPersonFiscalcode(fiscalCode.get(), year, includeDetails);
			} else {
				if (email.isPresent()) {
					logger.info("REST request to get ePAS Leaves by Person email: {}", email.get());
					epasLeavesList = epasLeavesService.getLeavesByPersonEmail(email.get(), year, includeDetails);
				} else {
					return ResponseUtil.wrapOrNotFound(Optional.of(epasLeavesList),
							HeaderUtil.createFailureAlert(applicationName, false, "", "", "Invalid parameter in call"));

				}
			}
		}
		logger.info("Retrieved List of Leaves: {}", epasLeavesList);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasLeavesList));
	}

	/**
	 * {@code GET /leaves/byOfficeAndYear} : get leaves by office and year.
	 *
	 * @param officeId       the office Id.
	 * @param year           the year.
	 * @param includeDetails a true or false flag that specifies whether or not to
	 *                       include absence details.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the list of EPAS Leaves, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/leaves/byOfficeAndYear")
	public ResponseEntity<List<EPASLeaves>> getEPASLeavesByOfficeCodeId(@RequestParam("officeId") String officeId,
			@RequestParam("year") String year, @RequestParam("includeDetails") String includeDetails) {
		logger.info("REST request to get ePAS Leaves by office: codeId={}, year={}", officeId, year);
		List<EPASLeaves> epasLeavesList = epasLeavesService.getLeavesByOfficeId(officeId, year, includeDetails);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasLeavesList));

	}

}
