package it.cnr.isti.epasmed.web.rest.epas;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Optional;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import io.github.jhipster.web.util.HeaderUtil;
import io.github.jhipster.web.util.ResponseUtil;
import it.cnr.isti.epasmed.epas.dto.EPASPersonsDTO;
import it.cnr.isti.epasmed.epas.model.EPASPersons;
import it.cnr.isti.epasmed.epas.service.EPASPersonsService;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;
import it.cnr.isti.epasmed.web.rest.errors.BadRequestAlertException;

@RestController
@RequestMapping("/api/epas")
public class EPASPersonsResource {

	private final Logger logger = LoggerFactory.getLogger(getClass());

	@Value("${jhipster.clientApp.name}")
	private String applicationName;

	private final EPASPersonsService epasPersonsService;

	public EPASPersonsResource(EPASPersonsService epasPersonsService) {
		this.epasPersonsService = epasPersonsService;

	}

	/**
	 * {@code GET /persons/:id} : get persons by id.
	 *
	 * @param id the id of the persons to find.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the EPAS Persons, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/persons/{id}")
	public ResponseEntity<EPASPersons> getEPASPersonsById(@PathVariable String id) {
		logger.info("REST request to get ePAS Persons by Id: {}", id);
		EPASPersons epasPersons = epasPersonsService.getById(id);
		logger.info("Retrieved Persons: {}", epasPersons);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasPersons));

	}

	/**
	 * {@code GET /persons/show} : get the persons by fiscal code.
	 *
	 * @param fc the fiscal code of the persons to find.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the ePAS Persons, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/persons/show")
	public ResponseEntity<EPASPersons> getEPASPersonsByFiscalCode(@RequestParam("fiscalCode") String fc) {
		logger.info("REST request to get ePAS Persons by fiscal code: {}", fc);
		EPASPersons epasPersons = epasPersonsService.getByFiscalCode(fc);
		logger.info("Retrieved Persons: {}", epasPersons);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasPersons));

	}

	/**
	 * {@code GET /persons} : get the persons list.
	 *
	 * @param officeId the office id.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the list of EPAS Persons, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/persons")
	public ResponseEntity<List<EPASPersons>> getEPASPersons(@RequestParam("officeId") String id) {
		logger.info("REST request to get ePAS Persons list for Office: {}", id);
		List<EPASPersons> epasPersonsList = epasPersonsService.getList(id);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasPersonsList));

	}

	/**
	 * {@code POST  /persons} : Creates a new persons.
	 * <p>
	 * Creates a new persons if the id is not already used.
	 *
	 * @param epasPersonsDTO the person to create.
	 * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with
	 *         body the new ePAS Person, or with status {@code 400 (Bad Request)} if
	 *         the id is already in use.
	 * @throws URISyntaxException       if the Location URI syntax is incorrect.
	 * @throws BadRequestAlertException {@code 400 (Bad Request)} if the id is
	 *                                  already in use.
	 */
	@PostMapping("/persons")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<EPASPersons> createEPASPersons(@Valid @RequestBody EPASPersonsDTO epasPersonsDTO)
			throws URISyntaxException {
		logger.debug("REST request to create EPAS Person: {}", epasPersonsDTO);

		if (epasPersonsDTO.getId() != null) {
			throw new BadRequestAlertException("A new ePAS Person already have an ID", "EPASPersons", "idexists");
		} else {
			EPASPersons createdEPASPersons = epasPersonsService.create(epasPersonsDTO);
			return ResponseEntity.created(new URI("/api/epas/persons/" + createdEPASPersons.getId()))
					.headers(HeaderUtil.createAlert(applicationName,
							"A ePAS Persons is created with identifier " + createdEPASPersons.getId(),
							createdEPASPersons.getFullname()))
					.body(createdEPASPersons);
		}
	}

	/**
	 * {@code PUT /persons/:id} : Updates an existing ePAS Person by id.
	 *
	 * @param epasPersonsDTO the new ePAS Person info.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the updated ePAS Persons.
	 * 
	 */
	@PutMapping("/persons/{id}")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<EPASPersons> updateEPASPersonsById(@PathVariable String id,
			@Valid @RequestBody EPASPersonsDTO epasPersonsDTO) {
		logger.debug("REST request to update ePAS Persons : {} by {}", id, epasPersonsDTO);
		epasPersonsService.updateById(id, epasPersonsDTO);
		EPASPersons updatedEPASPersons=epasPersonsService.getById(id);
		Optional<EPASPersons> oEPASPersons = Optional.of(updatedEPASPersons);

		return ResponseUtil.wrapOrNotFound(oEPASPersons, HeaderUtil.createAlert(applicationName,
				"A ePAS Person is updated with identifier " + updatedEPASPersons.getId(), updatedEPASPersons.getFullname()));
	}

	/**
	 * {@code PUT /persons/update} : Updates an existing ePAS Person by fiscal code.
	 *
	 * @param fc the fiscal code of the persons to update.
	 * @param epasPersonsDTO the new ePAS Person info.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the updated ePAS Persons.
	 * 
	 */
	@PutMapping("/persons/update")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<EPASPersons> updateEPASPersons(@RequestParam("fiscalCode") String fc,
			@Valid @RequestBody EPASPersonsDTO epasPersonsDTO) {
		logger.debug("REST request to update ePAS Persons : {}", epasPersonsDTO);
		epasPersonsService.updateByFiscalCode(fc, epasPersonsDTO);
		EPASPersons updatedEPASPersons=epasPersonsService.getByFiscalCode(fc);
		Optional<EPASPersons> oEPASPersons = Optional.of(updatedEPASPersons);

		return ResponseUtil.wrapOrNotFound(oEPASPersons, HeaderUtil.createAlert(applicationName,
				"A ePAS Person is updated with id " + updatedEPASPersons.getId(), updatedEPASPersons.getFullname()));
	}

	/**
	 * {@code DELETE /persons/:id} : delete ePAS Person by id.
	 *
	 * @param id the id of the ePAS Person to delete.
	 * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
	 */
	@DeleteMapping("/persons/{id}")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<Void> deleteEPASPersonById(@PathVariable String id) {
		logger.debug("REST request to delete ePAS Person by id: {}", id);
		epasPersonsService.deleteById(id);
		return ResponseEntity.noContent()
				.headers(HeaderUtil.createAlert(applicationName, "A Person is deleted with identifier " + id, id))
				.build();
	}

	/**
	 * {@code DELETE /persons/delete} : delete ePAS Person by fiscal code.
	 *
	 * @param id the fiscal code of the ePAS Person to delete.
	 * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
	 */
	@DeleteMapping("/persons/delete")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<Void> deleteEPASPersonByFiscalCode(@RequestParam("fiscalCode") String fc) {
		logger.debug("REST request to delete ePAS Person by fiscal code: {}", fc);
		epasPersonsService.deleteByFiscalCode(fc);
		return ResponseEntity.noContent()
				.headers(HeaderUtil.createAlert(applicationName, "A Person is deleted with fiscal code " + fc, fc))
				.build();
	}

}
