package it.cnr.isti.epasmed.web.rest.epas;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Optional;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import io.github.jhipster.web.util.HeaderUtil;
import io.github.jhipster.web.util.ResponseUtil;
import it.cnr.isti.epasmed.epas.dto.EPASStampingsDTO;
import it.cnr.isti.epasmed.epas.model.EPASStampings;
import it.cnr.isti.epasmed.epas.service.EPASStampingsService;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;
import it.cnr.isti.epasmed.web.rest.errors.BadRequestAlertException;

@RestController
@RequestMapping("/api/epas")
public class EPASStampingsResource {

	private final Logger logger = LoggerFactory.getLogger(getClass());

	@Value("${jhipster.clientApp.name}")
	private String applicationName;

	private final EPASStampingsService epasStampingsService;

	public EPASStampingsResource(EPASStampingsService epasStampingsService) {
		this.epasStampingsService = epasStampingsService;

	}

	/**
	 * {@code GET /stampings/:id} : get stampings by id.
	 *
	 * @param id the id of the stampings to find.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the EPAS Stampings, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/stampings/{id}")
	public ResponseEntity<EPASStampings> getEPASStampingsById(@PathVariable String id) {
		logger.info("REST request to get ePAS Stampings by Id: {}", id);
		EPASStampings epasStampings = epasStampingsService.getById(id);
		logger.info("Retrieved Stampings: {}", epasStampings);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasStampings));

	}

	/**
	 * {@code POST  /stampings} : Creates a new stampings.
	 * <p>
	 * Creates a new stampings.
	 *
	 * @param epasStampingsDTO the stamping to create.
	 * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with
	 *         body the new ePAS Stamping, or with status {@code 400 (Bad Request)} if
	 *         the id is already in use.
	 * @throws URISyntaxException       if the Location URI syntax is incorrect.
	 * @throws BadRequestAlertException {@code 400 (Bad Request)} if the id is
	 *                                  already in use.
	 */
	@PostMapping("/stampings")
	@PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
	public ResponseEntity<EPASStampings> createEPASStampings(@Valid @RequestBody EPASStampingsDTO epasStampingsDTO)
			throws URISyntaxException {
		logger.debug("REST request to create EPAS Stamping: {}", epasStampingsDTO);

		EPASStampings createdEPASStampings = epasStampingsService.create(epasStampingsDTO);
		return ResponseEntity.created(new URI("/api/epas/stampings/" + createdEPASStampings.getId()))
				.headers(HeaderUtil.createAlert(applicationName,
						"A ePAS Stampings is created with identifier " + createdEPASStampings.getId(),
						createdEPASStampings.getId()))
				.body(createdEPASStampings);

	}

}
