package it.cnr.isti.epasmed.web.rest.epas;

import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.put;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.env.Environment;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.context.junit.jupiter.EnabledIf;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import it.cnr.isti.epasmed.EpasmedApp;
import it.cnr.isti.epasmed.epas.dto.EPASChildrenDTO;
import it.cnr.isti.epasmed.epas.mapper.EPASChildrenMapper;
import it.cnr.isti.epasmed.epas.model.EPASChildren;
import it.cnr.isti.epasmed.epas.model.EPASOffice;
import it.cnr.isti.epasmed.epas.model.EPASPersons;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;
import it.cnr.isti.epasmed.web.rest.TestUtil;

/**
 * Integration tests for the {@link EPASChildrenResource} REST controller.
 */
@AutoConfigureMockMvc
@WithMockUser(authorities = AuthoritiesConstants.ADMIN)
@SpringBootTest(classes = EpasmedApp.class)
@EnabledIf("false")
public class EPASChildrenResourceIT {

	private final Logger log = LoggerFactory.getLogger(getClass());

	private static final String OFFICE_DEFAULT_ID = "1";
	private static final String OFFICE_DEFAULT_NAME = "ISTI - Pisa";
	private static final String OFFICE_DEFAULT_CODE = "074000";
	private static final String OFFICE_DEFAULT_CODEID = "225200";

	private static final String PERSON_DEFAULT_ID = "113";
	// private static final String PERSON_DEFAULT_NAME = "Giuseppe";
	// private static final String PERSON_DEFAULT_SURNAME = "Amato";
	private static final String PERSON_DEFAULT_FISCAL_CODE = "MTAGPP68D15D976W";
	private static final String PERSON_DEFAULT_EMAIL = "giuseppe.amato@cnr.it";
	// private static final String PERSON_DEFAULT_QUALIFICATION = "2";

	private static final String CHILDREN_DEFAULT_ID = "xxx";
	private static final String CHILDREN_DEFAULT_NAME = "childNomeTest1";
	private static final String CHILDREN_DEFAULT_SURNAME = "childCognomeTest1";
	private static final String CHILDREN_DEFAULT_FISCAL_CODE = "childCodicefiscaleTest1";

	private static final String CHILDREN_UPDATE_NAME = "childNomeTest2";

	@Autowired
	private MockMvc restEPASChildrenMockMvc;

	@Autowired
	private EPASChildrenMapper epasChildrenMapper;

	@Autowired
	private Environment environment;

	private EPASChildren epasChildren;
	private EPASChildrenDTO epasChildrenDTO;

	public static EPASOffice createOffice() {
		EPASOffice epasOffice = new EPASOffice();
		epasOffice.setId(OFFICE_DEFAULT_ID);
		epasOffice.setName(OFFICE_DEFAULT_NAME);
		epasOffice.setCode(OFFICE_DEFAULT_CODE);
		epasOffice.setCodeId(OFFICE_DEFAULT_CODEID);
		return epasOffice;
	}

	public static EPASChildren createTestChild() {
		EPASChildren epasChildren = new EPASChildren();
		epasChildren.setId(CHILDREN_DEFAULT_ID);
		epasChildren.setName(CHILDREN_DEFAULT_NAME);
		epasChildren.setSurname(CHILDREN_DEFAULT_SURNAME);
		epasChildren.setFiscalCode(CHILDREN_DEFAULT_FISCAL_CODE);
		epasChildren.setBornDate("2021-10-01");
		return epasChildren;
	}

	@BeforeEach
	public void initTest() {
		for (String profileName : environment.getActiveProfiles()) {
			log.info("Currently active profile - " + profileName);
		}
		log.info("System env - " + System.getenv("spring.profiles.active"));
		epasChildren = createTestChild();
		epasChildrenDTO = epasChildrenMapper.epasChildrenToEPASChildrenDTO(epasChildren);

	}

	@Test
	public void getChildrenById() throws Exception {
		restEPASChildrenMockMvc.perform(get("/api/epas/child/1")).andExpect(status().isOk());
	}

	@Test
	public void getChildrenByPersonId() throws Exception {
		restEPASChildrenMockMvc.perform(get("/api/epas/child/byPerson?id=" + PERSON_DEFAULT_ID))
				.andExpect(status().isOk());
	}

	@Test
	public void getChilrenByPersonFiscalCode() throws Exception {
		restEPASChildrenMockMvc.perform(get("/api/epas/child/byPerson?fiscalCode=" + PERSON_DEFAULT_FISCAL_CODE))
				.andExpect(status().isOk());
	}

	@Test
	public void getChildrenByPersonEmail() throws Exception {
		restEPASChildrenMockMvc.perform(get("/api/epas/child/byPerson?email=" + PERSON_DEFAULT_EMAIL))
				.andExpect(status().isOk());
	}

	@Test
	public void createChildren() throws Exception {
		restEPASChildrenMockMvc
				.perform(post("/api/epas/child").contentType(MediaType.APPLICATION_JSON)
						.content(TestUtil.convertObjectToJsonBytes(epasChildrenDTO)).with(csrf()))
				.andExpect(status().isCreated());

	}

	@Test
	public void istiAddChildren() {
		String userDirectory = System.getProperty("user.dir");
		log.info(userDirectory);
		List<EPASChildrenDTO> istiChildren = null;
		try (Stream<String> stream = Files
				.lines(Paths.get("src/test/resources/it/cnr/isti/epasmed/web/rest/epas/figliISTI.csv"))) {
			istiChildren = stream.skip(1).map(istiMapToChildren).collect(Collectors.toList());
		} catch (Exception e) {
			log.error(e.getLocalizedMessage(), e);
			return;
		}
		log.info("ISTI Children loaded");

		List<EPASPersons> epasPersons = null;
		try {
			MvcResult result = restEPASChildrenMockMvc.perform(get("/api/epas/persons?officeId=" + OFFICE_DEFAULT_ID))
					.andExpect(status().isOk()).andReturn();

			ObjectMapper mapper = new ObjectMapper();
			epasPersons = mapper.readValue(result.getResponse().getContentAsString(),
					new TypeReference<List<EPASPersons>>() {
					});

		} catch (Exception e) {
			log.error(e.getLocalizedMessage(), e);
			return;
		}

		log.info("EPAS Persons loaded");

		List<EPASChildrenDTO> createDTOList = new ArrayList<>();

		for (EPASChildrenDTO childrenDTO : istiChildren) {
			if (childrenDTO.getPersonId() != null && !childrenDTO.getPersonId().isEmpty()) {
				for (EPASPersons istip : epasPersons) {
					if (istip.getFiscalCode() != null && !istip.getFiscalCode().isEmpty()
							&& istip.getFiscalCode().compareTo(childrenDTO.getPersonId()) == 0) {
						List<EPASChildren> allReadyPresentChildren = new ArrayList<>();
						try {
							MvcResult result = restEPASChildrenMockMvc
									.perform(get("/api/epas/child/byPerson?fiscalCode=" + istip.getFiscalCode()))
									.andExpect(status().isOk()).andReturn();

							ObjectMapper mapper = new ObjectMapper();
							allReadyPresentChildren = mapper.readValue(result.getResponse().getContentAsString(),
									new TypeReference<List<EPASChildren>>() {
									});

						} catch (Exception e) {
							log.error(e.getLocalizedMessage(), e);
							return;
						}

						boolean allReadyPresentC = false;
						for (EPASChildren aec : allReadyPresentChildren) {
							if (aec.getSurname().compareToIgnoreCase(childrenDTO.getSurname()) == 0
									&& aec.getName().compareToIgnoreCase(childrenDTO.getName()) == 0) {
								allReadyPresentC = true;
								break;
							}
						}
						if (!allReadyPresentC) {
							childrenDTO.setPersonId(istip.getId());
							createDTOList.add(childrenDTO);
						}
						break;
					}
				}
			}
		}

		log.info("Children Create List builded");

		for (EPASChildrenDTO up : createDTOList) {
			try {
				restEPASChildrenMockMvc
						.perform(post("/api/epas/child").contentType(MediaType.APPLICATION_JSON)
								.content(TestUtil.convertObjectToJsonBytes(up)).with(csrf()))
						.andExpect(status().isOk());
			} catch (Exception e) {
				log.error(e.getLocalizedMessage(), e);
			}
		}
	}

	private Function<String, EPASChildrenDTO> istiMapToChildren = (line) -> {

		String[] p = line.split(",");// a CSV has comma separated lines

		EPASChildrenDTO child = new EPASChildrenDTO();

		// <-- this is the first column in the csv file
		if (p[0] != null && p[0].length() > 0) {
			String surname = p[0].substring(1, p[0].length() - 1);
			if (surname != null && !surname.isEmpty()) {
				child.setSurname(surname);
			}
		}

		if (p[1] != null && p[1].length() > 0) {
			String name = p[1].substring(1, p[1].length() - 1);
			if (name != null && !name.isEmpty()) {
				child.setName(name);
			}
		}

		if (p[2] != null && p[2].length() > 0) {
			String bornDate = p[2].substring(1, p[2].length() - 1);
			if (bornDate != null && !bornDate.isEmpty()) {
				child.setBornDate(bornDate);
			}
		}

		if (p[3] != null && p[3].length() > 0) {
			String externalId = p[3].substring(1, p[3].length() - 1);
			if (externalId != null && !externalId.isEmpty()) {
				child.setExternalId(externalId);
			}
		}

		if (p[4] != null && p[4].length() > 0) {
			String parentCodiceFiscale = p[4].substring(1, p[4].length() - 1);
			if (parentCodiceFiscale != null && !parentCodiceFiscale.isEmpty()) {
				child.setPersonId(parentCodiceFiscale);
			}
		}

		child.setFiscalCode("");
		return child;
	};

	@Test
	public void updateChildrenById() throws Exception {
		epasChildrenDTO.setName(CHILDREN_UPDATE_NAME);
		MvcResult mvcResult = restEPASChildrenMockMvc
				.perform(put("/api/epas/child/" + CHILDREN_DEFAULT_ID).contentType(MediaType.APPLICATION_JSON)
						.content(TestUtil.convertObjectToJsonBytes(epasChildrenDTO)).with(csrf()))
				.andExpect(status().isOk()).andReturn();
		ObjectMapper mapper = new ObjectMapper();

		EPASChildren epasChildren = mapper.readValue(mvcResult.getResponse().getContentAsByteArray(),
				EPASChildren.class);
		log.info("Updated EPAS Child by id: " + epasChildren);
	}

	@Test
	public void deleteChildrenById() throws Exception {
		restEPASChildrenMockMvc.perform(delete("/api/epas/child/" + CHILDREN_DEFAULT_ID).with(csrf()))
				.andExpect(status().is2xxSuccessful()).andReturn();
		log.info("Deleted EPAS Child by id: " + CHILDREN_DEFAULT_ID);
	}

}
