package it.cnr.isti.epasmed.web.rest.epas;

import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.put;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.env.Environment;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.context.junit.jupiter.EnabledIf;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import it.cnr.isti.epasmed.EpasmedApp;
import it.cnr.isti.epasmed.epas.dto.EPASPersonsDTO;
import it.cnr.isti.epasmed.epas.mapper.EPASPersonsMapper;
import it.cnr.isti.epasmed.epas.model.EPASOffice;
import it.cnr.isti.epasmed.epas.model.EPASPersons;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;
import it.cnr.isti.epasmed.web.rest.TestUtil;

/**
 * Integration tests for the {@link EPASPersonsResource} REST controller.
 */
@AutoConfigureMockMvc
@WithMockUser(authorities = AuthoritiesConstants.ADMIN)
@SpringBootTest(classes = EpasmedApp.class)
@EnabledIf("false")
public class EPASPersonsResourceIT {

	private final Logger log = LoggerFactory.getLogger(getClass());

	private static final String OFFICE_DEFAULT_ID = "1";
	private static final String OFFICE_DEFAULT_NAME = "ISTI - Pisa";
	private static final String OFFICE_DEFAULT_CODE = "074000";
	private static final String OFFICE_DEFAULT_CODEID = "225200";

	private static final String PERSON_DEFAULT_ID = "113";
	private static final String PERSON_DEFAULT_NAME = "nometest1";
	private static final String PERSON_DEFAULT_SURNAME = "cognometest1";
	private static final String PERSON_DEFAULT_FISCAL_CODE = "codicefiscaletes";
	private static final String PERSON_DEFAULT_EMAIL = "nometest1.cognometest1@test.it";
	private static final String PERSON_DEFAULT_QUALIFICATION = "3";

	private static final String PERSON_UPDATE_EMAIL = "nometest1.cognometest1@updated.it";
	private static final String PERSON_UPDATE_2_EMAIL = "nometest1.cognometest1@updated2.it";

	@Autowired
	private MockMvc restEPASPersonsMockMvc;

	@Autowired
	private EPASPersonsMapper epasPersonsMapper;

	@Autowired
	private Environment environment;

	private EPASPersons epasPersons;

	private EPASPersonsDTO epasPersonsDTO;

	/**
	 * Create a EPAS Persons.
	 *
	 * This is a static method
	 */
	public static EPASPersons createEPASPerson() {
		EPASPersons epasPersons = new EPASPersons();
		epasPersons.setName(PERSON_DEFAULT_NAME);
		epasPersons.setSurname(PERSON_DEFAULT_SURNAME);
		epasPersons.setFiscalCode(PERSON_DEFAULT_FISCAL_CODE);
		epasPersons.setEmail(PERSON_DEFAULT_EMAIL);
		epasPersons.setQualification(PERSON_DEFAULT_QUALIFICATION);
		epasPersons.setOffice(createOffice());
		return epasPersons;
	}

	public static EPASOffice createOffice() {
		EPASOffice epasOffice = new EPASOffice();
		epasOffice.setId(OFFICE_DEFAULT_ID);
		epasOffice.setName(OFFICE_DEFAULT_NAME);
		epasOffice.setCode(OFFICE_DEFAULT_CODE);
		epasOffice.setCodeId(OFFICE_DEFAULT_CODEID);
		return epasOffice;
	}

	@BeforeEach
	public void initTest() {
		for (String profileName : environment.getActiveProfiles()) {
			log.info("Currently active profile - " + profileName);
		}
		log.info("System env - " + System.getenv("spring.profiles.active"));
		epasPersons = createEPASPerson();
		epasPersonsDTO = epasPersonsMapper.epasPersonsToEPASPersonsDTO(epasPersons);
	}

	@Test
	public void getPersonById() throws Exception {
		restEPASPersonsMockMvc.perform(get("/api/epas/persons/"+PERSON_DEFAULT_ID)).andExpect(status().isOk());
	}
	
	@Test
	public void getPersonByFiscalCode() throws Exception {
		restEPASPersonsMockMvc.perform(get("/api/epas/persons/show?fiscalCode="+PERSON_DEFAULT_FISCAL_CODE)).andExpect(status().isOk());
	}
	
	@Test
	public void getPersonList() throws Exception {
		restEPASPersonsMockMvc.perform(get("/api/epas/persons?officeId=" + OFFICE_DEFAULT_ID))
				.andExpect(status().isOk());
	}

	@Test
	public void createPerson() throws Exception {
		restEPASPersonsMockMvc
				.perform(post("/api/epas/persons").contentType(MediaType.APPLICATION_JSON)
						.content(TestUtil.convertObjectToJsonBytes(epasPersonsDTO)).with(csrf()))
				.andExpect(status().isCreated());

	}

	@Test
	public void istiUpdatedPerson() {
		String userDirectory = System.getProperty("user.dir");
		log.info(userDirectory);
		List<EPASPersonsDTO> istiPersons = null;
		try (Stream<String> stream = Files
				.lines(Paths.get("src/test/resources/it/cnr/isti/epasmed/web/rest/epas/personaleISTI.csv"))) {
			istiPersons = stream.skip(1).map(istiMapToPerson).collect(Collectors.toList());
		} catch (Exception e) {
			log.error(e.getLocalizedMessage(), e);
			return;
		}
		log.info("ISTI Persons loaded");

		List<EPASPersons> epasPersons = null;
		try {
			MvcResult result = restEPASPersonsMockMvc.perform(get("/api/epas/persons?officeId=" + OFFICE_DEFAULT_ID))
					.andExpect(status().isOk()).andReturn();

			ObjectMapper mapper = new ObjectMapper();
			epasPersons = mapper.readValue(result.getResponse().getContentAsString(),
					new TypeReference<List<EPASPersons>>() {
					});

		} catch (Exception e) {
			log.error(e.getLocalizedMessage(), e);
			return;
		}

		log.info("EPAS Persons loaded");

		List<EPASPersons> updateList = new ArrayList<>();

		for (EPASPersons ep : epasPersons) {
			if (ep.getNumber() != null && !ep.getNumber().isEmpty()) {
				String number = ep.getNumber();
				boolean foundMatch = false;
				EPASPersonsDTO istiPers = null;
				for (EPASPersonsDTO istip : istiPersons) {
					if (istip.getNumber() != null && !istip.getNumber().isEmpty()
							&& istip.getNumber().compareTo(number) == 0) {
						foundMatch = true;
						istiPers = istip;
						break;
					}
				}
				if (foundMatch) {
					ep.setFiscalCode(istiPers.getFiscalCode());
					ep.setTelephone(istiPers.getTelephone());
					updateList.add(ep);
				}
			}
		}

		log.info("UpdateList created");

		List<EPASPersonsDTO> updateDTOList = epasPersonsMapper.epasPersonsToEPASPersonsDTOs(updateList);

		for (EPASPersonsDTO up : updateDTOList) {
			try {
				restEPASPersonsMockMvc
						.perform(put("/api/epas/persons/" + up.getId()).contentType(MediaType.APPLICATION_JSON)
								.content(TestUtil.convertObjectToJsonBytes(up)).with(csrf()))
						.andExpect(status().isOk());
			} catch (Exception e) {
				log.error(e.getLocalizedMessage(), e);
			}
		}
	}

	private Function<String, EPASPersonsDTO> istiMapToPerson = (line) -> {

		String[] p = line.split(",");// a CSV has comma separated lines

		EPASPersonsDTO pers = new EPASPersonsDTO();

		// <-- this is the first column in the csv file
		if (p[1] != null && p[1].length() > 0) {
			String surname = p[1].substring(1, p[1].length() - 1);
			if (surname != null && !surname.isEmpty()) {
				pers.setSurname(surname);
			}
		}

		if (p[2] != null && p[2].length() > 0) {
			String name = p[2].substring(1, p[2].length() - 1);
			if (name != null && !name.isEmpty()) {
				pers.setName(name);
			}
		}

		if (p[3] != null && p[3].length() > 0) {
			String matricola = p[3];
			if (matricola != null && !matricola.isEmpty()) {
				pers.setNumber(matricola);
			}
		}

		if (p[4] != null && p[4].length() > 0) {
			String fiscalCode = p[4].substring(1, p[4].length() - 1);
			if (fiscalCode != null && !fiscalCode.isEmpty()) {
				pers.setFiscalCode(fiscalCode);
			}
		}

		if (p[5] != null && p[5].length() > 0) {
			String telephone = p[5].substring(1, p[5].length() - 1);
			if (telephone != null && !telephone.isEmpty()) {
				pers.setTelephone(telephone);
			}
		}

		return pers;
	};

	@Test
	public void updatePersonById() throws Exception {
		epasPersonsDTO.setEmail(PERSON_UPDATE_EMAIL);
		MvcResult mvcResult = restEPASPersonsMockMvc
				.perform(put("/api/epas/persons/" + PERSON_DEFAULT_ID).contentType(MediaType.APPLICATION_JSON)
						.content(TestUtil.convertObjectToJsonBytes(epasPersonsDTO)).with(csrf()))
				.andExpect(status().isOk()).andReturn();
		ObjectMapper mapper = new ObjectMapper();

		EPASPersons epasPersons = mapper.readValue(mvcResult.getResponse().getContentAsByteArray(), EPASPersons.class);
		log.info("Updated EPAS Person by id: " + epasPersons);
	}

	@Test
	public void updatePersonByFiscalCode() throws Exception {
		epasPersonsDTO.setEmail(PERSON_UPDATE_2_EMAIL);
		MvcResult mvcResult = restEPASPersonsMockMvc
				.perform(put("/api/epas/persons/update?fiscalCode=" + PERSON_DEFAULT_FISCAL_CODE)
						.contentType(MediaType.APPLICATION_JSON)
						.content(TestUtil.convertObjectToJsonBytes(epasPersonsDTO)).with(csrf()))
				.andExpect(status().isOk()).andReturn();
		ObjectMapper mapper = new ObjectMapper();

		EPASPersons epasPersons = mapper.readValue(mvcResult.getResponse().getContentAsByteArray(), EPASPersons.class);
		log.info("Updated EPAS Person by fiscalcode: " + epasPersons);
	}

	@Test
	public void deletePersonById() throws Exception {
		restEPASPersonsMockMvc.perform(delete("/api/epas/persons/" + PERSON_DEFAULT_ID).with(csrf()))
				.andExpect(status().is2xxSuccessful()).andReturn();
		log.info("Deleted EPAS Person by id: " + PERSON_DEFAULT_ID);
	}

	@Test
	public void deletePersonByFiscalCode() throws Exception {
		restEPASPersonsMockMvc
				.perform(delete("/api/epas/persons/delete?fiscalCode=" + PERSON_DEFAULT_FISCAL_CODE).with(csrf()))
				.andExpect(status().is2xxSuccessful());
		log.info("Deleted EPAS Person by id: " + PERSON_DEFAULT_ID);
	}

}
