package it.cnr.isti.epasmed.web.rest.epas;

import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.io.FileWriter;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.env.Environment;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.context.junit.jupiter.EnabledIf;
import org.springframework.test.web.servlet.MockMvc;

import it.cnr.isti.epasmed.EpasmedApp;
import it.cnr.isti.epasmed.epas.dto.EPASStampingsDTO;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;
import it.cnr.isti.epasmed.web.rest.TestUtil;

/**
 * Integration tests for the {@link EPASStampingsResource} REST controller.
 */
@AutoConfigureMockMvc
@WithMockUser(authorities = AuthoritiesConstants.ADMIN)
@SpringBootTest(classes = EpasmedApp.class)
@EnabledIf("false")
public class EPASStampingsResourceIT {

	private static final Logger logger = LoggerFactory.getLogger(EPASStampingsResourceIT.class);

	// private static final String OFFICE_DEFAULT_ID = "1";
	// private static final String OFFICE_DEFAULT_NAME = "ISTI - Pisa";
	// private static final String OFFICE_DEFAULT_CODE = "074000";
	// private static final String OFFICE_DEFAULT_CODEID = "225200";

	private static final String STAMPING_DEFAULT_WAYTYPE = "in";
	private static final String STAMPING_DEFAULT_REASONTYPE = null;
	private static final String STAMPING_DEFAULT_DATETIME = "2021-02-12T08:20";
	private static final String STAMPING_DEFAULT_BADGENUMBER = "9802";
	private static final String STAMPING_DEFAULT_ZONE = null;
	private static final String STAMPING_DEFAULT_NOTE = "Timbratura test";
	private static final String STAMPING_DEFAULT_PLACE = null;
	private static final String STAMPING_DEFAULT_REASON = null;

	@Autowired
	private MockMvc restEPASStampingsMockMvc;

	@Autowired
	private Environment environment;

	private EPASStampingsDTO epasStampingsDTO;

	/**
	 * Create a EPASStampingsDTO.
	 *
	 * This is a static method
	 */
	public static EPASStampingsDTO createEPASStampingsDTO() {
		EPASStampingsDTO epasStampingsDTO = new EPASStampingsDTO();
		epasStampingsDTO.setWayType(STAMPING_DEFAULT_WAYTYPE);
		epasStampingsDTO.setReasonType(STAMPING_DEFAULT_REASONTYPE);
		epasStampingsDTO.setDateTime(STAMPING_DEFAULT_DATETIME);
		epasStampingsDTO.setBadgeNumber(STAMPING_DEFAULT_BADGENUMBER);
		epasStampingsDTO.setZone(STAMPING_DEFAULT_ZONE);
		epasStampingsDTO.setNote(STAMPING_DEFAULT_NOTE);
		epasStampingsDTO.setPlace(STAMPING_DEFAULT_PLACE);
		epasStampingsDTO.setReason(STAMPING_DEFAULT_REASON);
		return epasStampingsDTO;
	}

	@BeforeEach
	public void initTest() {
		for (String profileName : environment.getActiveProfiles()) {
			logger.info("Currently active profile - " + profileName);
		}
		logger.info("System env - " + System.getenv("spring.profiles.active"));
		epasStampingsDTO = createEPASStampingsDTO();
	}

	@Test
	public void getStampingById() throws Exception {
		restEPASStampingsMockMvc.perform(get("/api/epas/stampings/14")).andExpect(status().isOk());
	}

	@Test
	public void createStamping() throws Exception {

		restEPASStampingsMockMvc
				.perform(post("/api/epas/stampings").contentType(MediaType.APPLICATION_JSON)
						.content(TestUtil.convertObjectToJsonBytes(epasStampingsDTO)).with(csrf()))
				.andExpect(status().isCreated());

	}

	@Test
	public void istiUpdatedStampings() {
		String userDirectory = System.getProperty("user.dir");
		logger.info(userDirectory);
		List<EPASStampingsDTO> istiStampings = null;
		try (Stream<String> stream = Files
				.lines(Paths.get("src/test/resources/it/cnr/isti/epasmed/web/rest/epas/timbratureISTI20220701.csv"))) {
			istiStampings = stream.skip(1).map(istiMapToStampingsDTO).collect(Collectors.toList());
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage(), e);
			return;
		}
		logger.info("ISTI StampingsDTO loaded");

		for (EPASStampingsDTO esDTO : istiStampings) {
			if (esDTO.getBadgeNumber() != null && !esDTO.getBadgeNumber().isEmpty()) {
				try {
					restEPASStampingsMockMvc
							.perform(post("/api/epas/stampings").contentType(MediaType.APPLICATION_JSON)
									.content(TestUtil.convertObjectToJsonBytes(esDTO)).with(csrf()))
							.andExpect(status().is2xxSuccessful());
				} catch (Exception e) {
					logger.error(e.getLocalizedMessage(), e);
				}
			}
		}

		logger.info("Stampings created");

	}

	private Function<String, EPASStampingsDTO> istiMapToStampingsDTO = (line) -> {

		String[] s = line.split(",");// a CSV has comma separated lines

		EPASStampingsDTO stamping = new EPASStampingsDTO();

		// Badge = Matricola
		if (s[2] != null && s[2].length() > 0) {
			String badge = s[2].substring(1, s[2].length() - 1);
			if (badge != null && !badge.isEmpty()) {
				stamping.setBadgeNumber(badge);
			}
		}

		// Operazione
		if (s[3] != null && s[3].length() > 0) {
			String operazione = s[3].substring(1, s[3].length() - 1);
			if (operazione != null && !operazione.isEmpty()) {
				switch (operazione) {
				case "Entrata":
					stamping.setWayType("in");
					break;
				case "Uscita":
					stamping.setWayType("out");
					break;
				default:
					break;
				}

			}
		}

		// Data
		if (s[4] != null && s[4].length() > 0) {
			String d = s[4].substring(1, s[4].length() - 1);
			if (d != null && !d.isEmpty()) {
				stamping.setDateTime(d);
			}
		}

		// Causale Codice
		if (s[5] != null && s[5].length() > 0) {
			String causale = s[5].substring(1, s[5].length() - 1);
			if (causale != null && !causale.isEmpty()) {
				switch (causale) {
				case "1":
					stamping.setReasonType("motiviDiServizio");
					break;
				case "5":
					stamping.setReasonType("lavoroFuoriSede");
					break;
				default:
					break;
				}

			}
		}

		// Timbratura Altro Motivo
		if (s[9] != null && s[9].length() > 0) {
			String note = s[9].substring(1, s[9].length() - 1);
			if (note != null && !note.isEmpty()) {
				stamping.setNote(note);
			}
		}

		return stamping;
	};

	@Test
	public void addVirgolette() {
		String userDirectory = System.getProperty("user.dir");
		logger.info(userDirectory);
		List<String> list = new ArrayList<>();
		try (Stream<String> stream = Files
				.lines(Paths.get("src/test/resources/it/cnr/isti/epasmed/web/rest/epas/timbratureISTI20220701.csv"))) {
			list = stream.collect(Collectors.toList());
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage(), e);
			return;
		}

		try {
			FileWriter writer = new FileWriter(
					"src/test/resources/it/cnr/isti/epasmed/web/rest/epas/timbratureISTI20220701_2.csv");
			for (String str : list) {
				writer.write("\""+str+"\"" + System.lineSeparator());
			}
			writer.close();
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage(), e);
		}
		logger.info("ISTI StampingsDTO updated");

	}

}
