package it.cnr.isti.epasmed.web.rest.sync;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.sql.Timestamp;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeFormatterBuilder;
import java.time.format.DateTimeParseException;
import java.time.temporal.ChronoUnit;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.env.Environment;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.context.junit.jupiter.EnabledIf;
import org.springframework.test.web.servlet.MockMvc;

import it.cnr.isti.epasmed.EpasmedApp;
import it.cnr.isti.epasmed.domain.PersOrario;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;

/**
 * Integration tests for the {@link SyncResource} REST controller.
 */
@AutoConfigureMockMvc
@WithMockUser(authorities = AuthoritiesConstants.ADMIN)
@SpringBootTest(classes = EpasmedApp.class)
@EnabledIf("false")
public class SyncResourceIT {

	private static final Logger logger = LoggerFactory.getLogger(SyncResourceIT.class);

	private final Logger log = LoggerFactory.getLogger(getClass());
	private static final String YEAR = "2022";
	private static final String MONTH = "10";
	private static final String PERSON_DEFAULT_FISCAL_CODE = "CSTDTL57L65A864M";

	@Autowired
	private MockMvc restSyncMockMvc;

	@Autowired
	private Environment environment;

	@BeforeEach
	public void initTest() {
		for (String profileName : environment.getActiveProfiles()) {
			log.info("Currently active profile - " + profileName);
		}
		log.info("System env - " + System.getenv("spring.profiles.active"));
	}

	@Test
	public void syncReads() throws Exception {
		restSyncMockMvc.perform(get("/api/sync/reads")).andExpect(status().is2xxSuccessful());
	}

	@Test
	public void syncWrites() throws Exception {
		restSyncMockMvc.perform(get("/api/sync/writes?year=" + YEAR + "&month=" + MONTH))
				.andExpect(status().is2xxSuccessful());
	}

	@Test
	public void syncTest() throws Exception {
		restSyncMockMvc.perform(get("/api/sync/test?year=" + YEAR + "&month=" + MONTH))
				.andExpect(status().is2xxSuccessful());
	}

	@Test
	public void syncWritesOrario() throws Exception {
		restSyncMockMvc.perform(get("/api/sync/writesOrario")).andExpect(status().is2xxSuccessful());
	}

	@Test
	public void syncWritesTimeCards() throws Exception {
		restSyncMockMvc.perform(get("/api/sync/writesTimeCards?year=" + YEAR + "&month=" + MONTH))
				.andExpect(status().is2xxSuccessful());
	}

	@Test
	public void syncWritesSingleTimeCards() throws Exception {
		restSyncMockMvc.perform(get("/api/sync/writesSingleTimeCards?year=" + YEAR + "&month=" + MONTH + "&fiscalCode="
				+ PERSON_DEFAULT_FISCAL_CODE));
	}

	@Test
	public void syncWritesOffSiteWorks() throws Exception {
		restSyncMockMvc.perform(get("/api/sync/writesOffSiteWorks?year=" + YEAR + "&month=" + MONTH));
	}

	@Test
	public void syncWritesLeaves() throws Exception {
		restSyncMockMvc.perform(get("/api/sync/writesLeaves?year=" + YEAR + "&month=" + MONTH));
	}

	@Test
	public void syncWritesPersonWorkingTime() throws Exception {
		restSyncMockMvc.perform(get("/api/sync/writesPersonWorkingTime?year=" + YEAR + "&month=" + MONTH));
	}

	@Test
	public void testDataMod() throws Exception {
		// "2021-02-03T09:49:05.231072"
		DateTimeFormatter formatter = new DateTimeFormatterBuilder().parseCaseInsensitive()
				.append(DateTimeFormatter.ISO_LOCAL_DATE_TIME).optionalStart().appendPattern(".SSSSSS").optionalEnd()
				.optionalStart().appendZoneOrOffsetId().optionalEnd().toFormatter();
		// DateTimeFormatter formatterDataMod = DateTimeFormatter.ofPattern("yyyy-MM-dd
		// HH:mm:ss");

		Timestamp dataMod;
		try {
			LocalDateTime dMod = LocalDateTime.parse("2021-02-03T19:49:05.231072", formatter);
			dMod = dMod.truncatedTo(ChronoUnit.SECONDS);
			// dMod= LocalDateTime.parse(dMod.format(formatterDataMod));
			// dataMod = Timestamp.valueOf(dMod.format(formatterDataMod));
			dataMod = Timestamp.valueOf(dMod);

		} catch (IllegalArgumentException | DateTimeParseException e) {
			logger.error("Invalid stamping data format: {}", e.getLocalizedMessage(), e);
			return;
		}

		logger.info("Result: {}", dataMod);
	}

	@Test
	public void testLocalData() throws Exception {

		LocalDateTime data = LocalDateTime.now();
		
		logger.info("Data: {}", data);
		logger.info("Year: {}", data.getYear());
		logger.info("Month: {}", data.getMonth());
		logger.info("Month Value: {}", data.getMonthValue());
		
		
		
	}
	
	
	@Test
	public void testSamePersOrario() throws Exception {

		PersOrario persOrario=new PersOrario();
		persOrario.setId(72);
		persOrario.setIdPersona(72);
		persOrario.setCf("VDCLCU87A64D086T");
		persOrario.setDal(LocalDate.of(2019,07,1));
		persOrario.setAl(null);
		persOrario.setDescrizione("Normale");
		persOrario.setLun(432);
		persOrario.setMar(432);
		persOrario.setMer(432);
		persOrario.setGio(432);
		persOrario.setVen(432);
		persOrario.setSab(0);
		persOrario.setPercentuale(100);
		persOrario.setTurno("NO");
		persOrario.setOreTurno(0);
		persOrario.setFestivo("SI");
		persOrario.setNotturno("NO");
		persOrario.setDataMod(LocalDateTime.parse("2022-10-25T03:00:01"));
		logger.info("PersOrario: {}", persOrario);
		
		PersOrario persOrarioDTO=new PersOrario();
		persOrarioDTO.setId(72);
		persOrarioDTO.setIdPersona(72);
		persOrarioDTO.setCf("VDCLCU87A64D086T");
		persOrarioDTO.setDal(LocalDate.of(2019,07,1));
		persOrarioDTO.setAl(null);
		persOrarioDTO.setDescrizione("Normale");
		persOrarioDTO.setLun(432);
		persOrarioDTO.setMar(432);
		persOrarioDTO.setMer(432);
		persOrarioDTO.setGio(432);
		persOrarioDTO.setVen(432);
		persOrarioDTO.setSab(0);
		persOrarioDTO.setPercentuale(100);
		persOrarioDTO.setTurno("NO");
		persOrarioDTO.setOreTurno(0);
		persOrarioDTO.setFestivo("SI");
		persOrarioDTO.setNotturno("NO");
		persOrarioDTO.setDataMod(LocalDateTime.parse("2022-10-25T11:23:30"));
		logger.info("PersOrarioDTO: {}", persOrarioDTO);
		
		logger.info("Is same: {}",persOrario.same(persOrarioDTO));
	}
	
	
	

}
